/* Scrutari - Copyright (c) 2009-2015 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.servlets;

import fr.exemole.sctserver.api.SctContext;
import fr.exemole.sctserver.request.json.JsonProducerFactory;
import fr.exemole.sctserver.servlets.admin.AdminRequest;
import fr.exemole.sctserver.servlets.explore.ExploreRequest;
import fr.exemole.sctserver.servlets.export.ExportRequest;
import fr.exemole.sctserver.servlets.feed.FeedRequest;
import fr.exemole.sctserver.servlets.global.GlobalRequest;
import java.io.IOException;
import javax.servlet.ServletException;
import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import net.mapeadores.util.json.JsonProducer;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.servlets.HttpServletRequestMap;
import net.mapeadores.util.servlets.ResponseHandler;
import net.mapeadores.util.servlets.handlers.JsonResponseHandler;


/**
 *
 * @author Vincent Calame
 */
public class ScrutariServlet extends HttpServlet {

    @Override
    public void doGet(HttpServletRequest requete, HttpServletResponse reponse) throws IOException, ServletException {
        process(requete, reponse);
    }

    @Override
    public void doPost(HttpServletRequest requete, HttpServletResponse reponse) throws IOException, ServletException {
        process(requete, reponse);
    }

    private void process(HttpServletRequest request, HttpServletResponse response) throws IOException, ServletException {
        String pathInfo = request.getPathInfo();
        if (pathInfo == null) {
            pathInfo = "/";
        }
        int idx = pathInfo.indexOf('/', 2);
        if (idx != -1) {
            String engineName = pathInfo.substring(1, idx);
            String sctserverPathInfo = pathInfo.substring(idx + 1);
            processSctServerRequest(engineName, sctserverPathInfo, request, response);
        } else {
            processGlobalRequest(pathInfo, request, response);
        }
    }

    private void processGlobalRequest(String path, HttpServletRequest request, HttpServletResponse response) throws IOException {
        ScrutariContext scrutariContext = ScrutariContext.getScrutariContext(getServletContext());
        StringBuffer requestURLString = request.getRequestURL();
        if (path.length() == 1) {
            if (requestURLString.charAt(requestURLString.length() - 1) != '/') {
                response.sendRedirect(requestURLString.append('/').toString());
                return;
            }
        }
        ResponseHandler responseHandler = null;
        if (path.endsWith(".css")) {
            responseHandler = ResponseHandlerFactory.getCssHandler(path);
        } else if (path.endsWith(".png")) {
            responseHandler = ResponseHandlerFactory.getPngHandler(path);
        } else {
            GlobalRequest globalRequest = GlobalRequest.init(path, scrutariContext);
            if (globalRequest == null) {
                if (scrutariContext.getSctContext(path) != null) {
                    response.sendRedirect(requestURLString.append('/').toString());
                    return;
                }
            } else {
                responseHandler = globalRequest.getResponsePrinter();
            }
        }
        handleResponse(request, response, responseHandler);
    }

    private void processSctServerRequest(String engineName, String sctserverPathInfo, HttpServletRequest request, HttpServletResponse response) throws IOException {
        ScrutariContext scrutariContext = ScrutariContext.getScrutariContext(getServletContext());
        SctContext sctContext = scrutariContext.getSctContext(engineName);
        if (sctContext == null) {
            printNotFound(request, response);
            return;
        }
        if (sctserverPathInfo.length() == 0) {
            response.sendError(HttpServletResponse.SC_FORBIDDEN);
            return;
        }
        int idx = sctserverPathInfo.indexOf('/');
        if (idx == -1) {
            String minuscule = sctserverPathInfo.toLowerCase();
            if (minuscule.equals("json")) {
                printJSon(request, response, sctContext);
            } else {
                printNotFound(request, response);
            }
        } else {
            String servletPath = sctserverPathInfo.substring(0, idx);
            sctserverPathInfo = sctserverPathInfo.substring(idx);
            ResponseHandler responseHandler = null;
            RequestMap requestMap = new HttpServletRequestMap(request);
            if (servletPath.equals("admin")) {
                AdminRequest adminRequest = AdminRequest.init(sctContext, sctserverPathInfo, requestMap);
                if (adminRequest != null) {
                    responseHandler = adminRequest.getResponseHandler();
                }
            } else if (servletPath.equals("explore")) {
                ExploreRequest exploreRequest = ExploreRequest.init(sctContext, sctserverPathInfo, requestMap);
                if (exploreRequest != null) {
                    responseHandler = exploreRequest.getResponseHandler();
                }
            } else if (servletPath.equals("feed")) {
                FeedRequest feedRequest = FeedRequest.init(sctContext, sctserverPathInfo, requestMap);
                if (feedRequest != null) {
                    responseHandler = feedRequest.getResponseHandler();
                }
            } else if (servletPath.equals("css")) {
                responseHandler = ResponseHandlerFactory.getCssHandler(sctserverPathInfo);
            } else if (servletPath.equals("js")) {
                responseHandler = ResponseHandlerFactory.getJsHandler(sctserverPathInfo);
            } else if (servletPath.equals("logs")) {
                responseHandler = ResponseHandlerFactory.getLogsHandler(sctserverPathInfo, sctContext.getSctServer().getSctServerStorage().getLogStorage());
            } else if (servletPath.equals("export")) {
                ExportRequest exportRequest = ExportRequest.init(sctContext, sctserverPathInfo, requestMap);
                if (exportRequest != null) {
                    responseHandler = exportRequest.getResponseHandler();
                }
            }
            handleResponse(request, response, responseHandler);
        }
    }

    private void handleResponse(HttpServletRequest request, HttpServletResponse response, ResponseHandler responseHandler) throws IOException {
        if (responseHandler == null) {
            printNotFound(request, response);
            return;
        }
        long lastModified = responseHandler.getLastModified();
        if (lastModified == ResponseHandler.LASTMODIFIED_NOCACHE) {
            response.setHeader("Cache-Control", "no-store");
        } else if (lastModified > 0) {
            response.setDateHeader("Last-Modified", lastModified);
            try {
                long since = request.getDateHeader("If-Modified-Since");
                if (since >= lastModified) {
                    response.setStatus(HttpServletResponse.SC_NOT_MODIFIED);
                    return;
                }
            } catch (IllegalArgumentException iae) {
            }
        }
        responseHandler.handleResponse(response);
    }

    private void printJSon(HttpServletRequest request, HttpServletResponse response, SctContext sctContext) throws IOException {
        HttpServletRequestMap requestMap = new HttpServletRequestMap(request);
        response.setHeader("Access-Control-Allow-Origin", "*");
        JsonProducer jsonProducer = JsonProducerFactory.getJsonProducer(sctContext.getSctServer().getScrutariSession(), requestMap);
        JsonResponseHandler jsonResponseHandler = new JsonResponseHandler(jsonProducer, requestMap.getParameter("callback"));
        jsonResponseHandler.handleResponse(response);
    }

    private void printNotFound(HttpServletRequest request, HttpServletResponse response) throws IOException {
        response.sendError(HttpServletResponse.SC_NOT_FOUND, request.getPathInfo());
    }

}
