/* ScrutariLib_DB - Copyright (c) 2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.db.tools;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.db.api.caches.LexieCacheWriter;
import net.scrutari.db.tools.lexies.LexieOccurrencesMapBuilder;
import net.scrutari.db.tools.lexies.LexieUnitBuilder;
import net.scrutari.lexie.FicheLexification;
import net.scrutari.lexie.LexieOccurrencesList;
import net.scrutari.lexie.MotcleLexification;
import net.scrutari.lexie.ScrutariLexieUnit;
import net.scrutari.lexie.ScrutariLexieUnitList;


/**
 *
 * @author Vincent Calame
 */
public final class LexificationEngine {

    private final LexieOccurrencesMapBuilder ficheLexieOccurrencesMapBuilder;
    private final LexieOccurrencesMapBuilder motcleLexieOccurrencesMapBuilder;
    private final LexieUnitBuilder lexieUnitBuilder;
    private final LexieCacheWriter lexieCacheWriter;

    private LexificationEngine(AttributeKeyCoder attributeKeyCoder, LexieCacheWriter lexieCacheWriter) {
        this.lexieCacheWriter = lexieCacheWriter;
        lexieUnitBuilder = new LexieUnitBuilder();
        ficheLexieOccurrencesMapBuilder = new LexieOccurrencesMapBuilder(attributeKeyCoder, lexieUnitBuilder, true);
        motcleLexieOccurrencesMapBuilder = new LexieOccurrencesMapBuilder(attributeKeyCoder, lexieUnitBuilder, false);
    }

    private void newCorpus(int codecorpus) {
        checkLexieOccurrencesMap(ficheLexieOccurrencesMapBuilder);
        ficheLexieOccurrencesMapBuilder.setCurrentSubsetCode(codecorpus);
    }

    private void initFicheData(FicheData ficheData) {
        FicheLexification ficheLexification = ficheLexieOccurrencesMapBuilder.initFicheData(ficheData);
        lexieCacheWriter.cacheFicheLexification(ficheData.getCodeFiche(), ficheLexification);
    }

    private void newThesaurus(int codethesaurus) {
        checkLexieOccurrencesMap(motcleLexieOccurrencesMapBuilder);
        motcleLexieOccurrencesMapBuilder.setCurrentSubsetCode(codethesaurus);
    }

    private void initMotcleData(MotcleData motcleData) {
        MotcleLexification motcleLexification = motcleLexieOccurrencesMapBuilder.initMotcleData(motcleData);
        lexieCacheWriter.cacheMotcleLexification(motcleData.getCodeMotcle(), motcleLexification);
    }

    private LexieCacheReaderFactory end() {
        checkLexieOccurrencesMap(ficheLexieOccurrencesMapBuilder);
        checkLexieOccurrencesMap(motcleLexieOccurrencesMapBuilder);
        Collection<ScrutariLexieUnit> coll = lexieUnitBuilder.getLexieUnitList();
        Map<Lang, InternalScrutariLexieUnitList> unitByLangMap = new HashMap<Lang, InternalScrutariLexieUnitList>();
        for (ScrutariLexieUnit scrutariLexieUnit : coll) {
            Lang lang = scrutariLexieUnit.getLang();
            InternalScrutariLexieUnitList scrutariLexieUnitList = unitByLangMap.get(lang);
            if (scrutariLexieUnitList == null) {
                scrutariLexieUnitList = new InternalScrutariLexieUnitList();
                unitByLangMap.put(lang, scrutariLexieUnitList);
            }
            scrutariLexieUnitList.addScrutariLexieUnit(scrutariLexieUnit);
        }
        for (Map.Entry<Lang, InternalScrutariLexieUnitList> entry : unitByLangMap.entrySet()) {
            lexieCacheWriter.cacheScrutariLexieUnitList(entry.getKey(), entry.getValue());
        }
        return lexieCacheWriter.endCacheWrite();
    }

    private void checkLexieOccurrencesMap(LexieOccurrencesMapBuilder lexieOccurrencesMapBuilder) {
        int currentSubsetCode = lexieOccurrencesMapBuilder.getCurrentSubsetCode();
        Map<Long, LexieOccurrencesList> map = lexieOccurrencesMapBuilder.popLexieOccurencesListMap();
        for (Map.Entry<Long, LexieOccurrencesList> entry : map.entrySet()) {
            long lg = entry.getKey();
            LexieOccurrencesList lexieOccurrencesList = entry.getValue();
            lexieCacheWriter.cacheLexieOccurrencesList(lg, currentSubsetCode, lexieOccurrencesList);
        }
    }

    public static LexieCacheReaderFactory cacheLexification(DataAccess dataAccess, AttributeKeyCoder attributeKeyCoder, LexieCacheWriter lexieCacheWriter) {
        LexificationEngine engine = new LexificationEngine(attributeKeyCoder, lexieCacheWriter);
        int corpusCount = dataAccess.getCorpusCount();
        for (int i = 0; i < corpusCount; i++) {
            CorpusData corpusData = dataAccess.getCorpusDataByIndex(i);
            engine.newCorpus(corpusData.getCodeCorpus());
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                FicheData ficheData = dataAccess.getFicheDataByCode(corpusData.getCodeFiche(j));
                engine.initFicheData(ficheData);
            }
        }
        int thesaurusCount = dataAccess.getThesaurusCount();
        for (int i = 0; i < thesaurusCount; i++) {
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByIndex(i);
            engine.newThesaurus(thesaurusData.getCodeThesaurus());
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                MotcleData motcleData = dataAccess.getMotcleDataByCode(thesaurusData.getCodeMotcle(j));
                engine.initMotcleData(motcleData);
            }
        }
        return engine.end();
    }


    private static class InternalScrutariLexieUnitList implements ScrutariLexieUnitList {

        private final List<ScrutariLexieUnit> list = new ArrayList<ScrutariLexieUnit>();

        private InternalScrutariLexieUnitList() {

        }

        @Override
        public int getScrutariLexieUnitCount() {
            return list.size();
        }

        @Override
        public ScrutariLexieUnit getScrutariLexieUnit(int i) {
            return list.get(i);
        }

        private void addScrutariLexieUnit(ScrutariLexieUnit scrutariLexieUnit) {
            list.add(scrutariLexieUnit);
        }

    }

}
