/* ScrutariLib_SearchEngine - Copyright (c) 2010-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.text.SubstringPosition;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.IndexationCodeList;
import net.scrutari.searchengine.api.pertinence.FichePertinence;
import net.scrutari.searchengine.api.pertinence.MotclePertinence;
import net.scrutari.searchengine.api.lexieseek.LexieSeekHandler;
import net.scrutari.searchengine.api.options.ComputedOptions;


/**
 *
 * @author Vincent Calame
 */
class SearchEngine implements LexieSeekHandler {

    private int operandMaxNumber = 0;
    private final Map<Integer, InternalFichePertinence> fichePertinenceMap = new HashMap<Integer, InternalFichePertinence>();
    private final Map<Integer, InternalMotclePertinence> motclePertinenceMap = new HashMap<Integer, InternalMotclePertinence>();
    private List<MotclePertinence> motclePertinenceList = new ArrayList<MotclePertinence>();
    private final int[] acceptOperandArray;

    SearchEngine(int operandMaxNumber, int[] acceptOperandArray) {
        this.operandMaxNumber = operandMaxNumber;
        this.acceptOperandArray = acceptOperandArray;
    }

    @Override
    public void addAlineaOccurrence(int codefiche, int codecorpus, int operandNumber, int alineaInteger, SubstringPosition sourcePosition) {
        InternalFichePertinence fichePertinence = fichePertinenceMap.get(codefiche);
        if (fichePertinence == null) {
            fichePertinence = new InternalFichePertinence(codefiche, codecorpus, operandMaxNumber);
            fichePertinenceMap.put(codefiche, fichePertinence);
        }
        fichePertinence.addOccurrence(operandNumber, alineaInteger, sourcePosition);
    }

    @Override
    public void addLangOccurrence(int codemotcle, int codethesaurus, int operandNumber, Lang lang, SubstringPosition sourcePosition, float coverRate) {
        InternalMotclePertinence motclePertinence = (InternalMotclePertinence) motclePertinenceMap.get(codemotcle);
        if (motclePertinence == null) {
            motclePertinence = new InternalMotclePertinence(codemotcle, codethesaurus, operandMaxNumber);
            motclePertinenceMap.put(codemotcle, motclePertinence);
            motclePertinenceList.add(motclePertinence);
        }
        motclePertinence.addOccurrence(operandNumber, lang, sourcePosition, coverRate);
    }

    FichePertinence getFichePertinence(int codefiche) {
        return fichePertinenceMap.get(codefiche);
    }

    List<FichePertinence> getFichePertinenceList() {
        List<FichePertinence> list = new ArrayList<FichePertinence>();
        list.addAll(fichePertinenceMap.values());
        return list;
    }

    List<MotclePertinence> getMotclePertinenceList() {
        return motclePertinenceList;
    }

    void checkMotclePertinence(DataAccess dataAccess, ComputedOptions computedOptions) {
        int[] codecorpusArray = computedOptions.getCodecorpusArray();
        Lang[] langArray = computedOptions.getLangArray();
        boolean allLang = computedOptions.isAllLang();
        int corpusCount = codecorpusArray.length;
        List<MotclePertinence> newMotclePertinenceList = new ArrayList<MotclePertinence>();
        for (InternalMotclePertinence motclePertinence : motclePertinenceMap.values()) {
            float[] motcleNoteArray = motclePertinence.checkMotcleScoreArray(acceptOperandArray);
            int codemotcle = motclePertinence.getCodemotcle();
            boolean isUsed = false;
            for (int k = 0; k < corpusCount; k++) {
                int codecorpus = codecorpusArray[k];
                IndexationCodeList indexationCodeList = dataAccess.getIndexationCodeList(codemotcle, codecorpus);
                int ficheCount = indexationCodeList.getCodeCount();
                for (int l = 0; l < ficheCount; l++) {
                    int codefiche = indexationCodeList.getCode(l);
                    InternalFichePertinence fichePertinence = fichePertinenceMap.get(codefiche);
                    if (fichePertinence == null) {
                        if (!computedOptions.acceptFiche(codefiche)) {
                            continue;
                        }
                        boolean add = false;
                        if (allLang) {
                            add = true;
                        } else {
                            Lang lang = dataAccess.getFicheLang(codefiche);
                            boolean contains = false;
                            int langLength = langArray.length;
                            for (int j = 0; j < langLength; j++) {
                                if (langArray[j].equals(lang)) {
                                    contains = true;
                                    break;
                                }
                            }
                            if (contains) {
                                add = true;
                            }
                        }
                        if (add) {
                            fichePertinence = new InternalFichePertinence(codefiche, codecorpus, operandMaxNumber);
                            fichePertinenceMap.put(codefiche, fichePertinence);
                        } else {
                            continue;
                        }
                    }
                    isUsed = true;
                    fichePertinence.addMotcleScore(motcleNoteArray);
                    if (motclePertinence.isWithAccept()) {
                        fichePertinence.addMotcle(codemotcle);
                    }
                }
            }
            if ((isUsed) && (motclePertinence.isWithAccept())) {
                newMotclePertinenceList.add(motclePertinence);
            }
        }
        this.motclePertinenceList = newMotclePertinenceList;
    }


    private static class InternalFichePertinence implements FichePertinence {

        private final int codefiche;
        private final int codecorpus;
        private final int[] alineaIntegerArray;
        private final float[] motcleScoreArray;
        private final SubstringPosition[] substringPositionArray;
        private final List<Integer> codemotcleList = new ArrayList<Integer>();

        private InternalFichePertinence(int codefiche, int codecorpus, int operandMaxNumber) {
            this.codefiche = codefiche;
            this.codecorpus = codecorpus;
            this.alineaIntegerArray = new int[operandMaxNumber];
            this.motcleScoreArray = new float[operandMaxNumber];
            for (int i = 0; i < operandMaxNumber; i++) {
                alineaIntegerArray[i] = -1;
                motcleScoreArray[i] = -1;
            }
            this.substringPositionArray = new SubstringPosition[operandMaxNumber];
        }

        private void addOccurrence(int operandNumber, int alineaInteger, SubstringPosition sourcePosition) {
            int idx = operandNumber - 1;
            if (sourcePosition == null) {
                alineaIntegerArray[idx] = -2;
                substringPositionArray[idx] = null;
            } else {
                int currentAlineaInteger = alineaIntegerArray[idx];
                if ((currentAlineaInteger == -1) || (currentAlineaInteger > alineaInteger)) {
                    alineaIntegerArray[idx] = alineaInteger;
                    substringPositionArray[idx] = sourcePosition;
                } else if (currentAlineaInteger == alineaInteger) {
                    if (sourcePosition.getBeginIndex() < substringPositionArray[idx].getBeginIndex()) {
                        substringPositionArray[idx] = sourcePosition;
                    }
                }
            }
        }

        @Override
        public short getOperandState(int operandNumber) {
            int index = operandNumber - 1;
            int alineaInteger = alineaIntegerArray[index];
            if (alineaInteger > 0) {
                return FichePertinence.OCCURRENCE_STATE;
            }
            float motcleScore = motcleScoreArray[index];
            if (motcleScore > 0) {
                return FichePertinence.OCCURRENCE_STATE;
            }
            if ((alineaInteger == -2) || (motcleScore < -1)) {
                return FichePertinence.NOPERTINENT_STATE;
            }
            return FichePertinence.NO_OCCURRENCE_STATE;
        }

        @Override
        public int getCodefiche() {
            return codefiche;
        }

        @Override
        public int getCodecorpus() {
            return codecorpus;
        }

        @Override
        public int getAlineaInteger(int operandNumber) {
            int alineaInteger = alineaIntegerArray[operandNumber - 1];
            if (alineaInteger < 0) {
                return -1;
            }
            return alineaInteger;
        }

        @Override
        public SubstringPosition getSubstringPosition(int operandNumber) {
            return substringPositionArray[operandNumber - 1];
        }

        @Override
        public int[] getAcceptCodemotcleArray() {
            int size = codemotcleList.size();
            int[] result = new int[size];
            for (int i = 0; i < size; i++) {
                result[i] = codemotcleList.get(i);
            }
            return result;
        }

        @Override
        public float getMotcleScore(int operandNumber) {
            float score = motcleScoreArray[operandNumber - 1];
            if (score < 0) {
                return 0;
            }
            if (score > 1) {
                return 1;
            }
            return score;
        }

        private void addMotcleScore(float[] array) {
            int length = array.length;
            for (int i = 0; i < length; i++) {
                float current = motcleScoreArray[i];
                float newNote = array[i];
                if (newNote > 0) {
                    if ((current < 0) || (newNote > current)) {
                        motcleScoreArray[i] = newNote;
                    }
                } else if (newNote < -1) {
                    if (current < 0) {
                        motcleScoreArray[i] = newNote;
                    }
                }
            }
        }

        private void addMotcle(int codemotcle) {
            codemotcleList.add(codemotcle);
        }

    }


    private static class InternalMotclePertinence implements MotclePertinence {

        private final int codemotcle;
        private final int codethesaurus;
        private final int operandMaxNumber;
        private final List<InternalLangOccurrence> langOccurrenceList = new ArrayList<InternalLangOccurrence>();
        private final int[] operandStateArray;
        private boolean withAccept;

        private InternalMotclePertinence(int codemotcle, int codethesaurus, int operandMaxNumber) {
            this.codemotcle = codemotcle;
            this.codethesaurus = codethesaurus;
            this.operandMaxNumber = operandMaxNumber;
            this.operandStateArray = new int[operandMaxNumber];
        }

        @Override
        public int getCodemotcle() {
            return codemotcle;
        }

        @Override
        public int getCodethesaurus() {
            return codethesaurus;
        }

        @Override
        public int getLangOccurrenceCount() {
            return langOccurrenceList.size();
        }

        private void addOccurrence(int operandNumber, Lang lang, SubstringPosition sourcePosition, float coverRate) {
            int index = operandNumber - 1;
            if (sourcePosition != null) {
                InternalLangOccurrence langOccurrence = getOrCreateLangOccurrence(lang);
                langOccurrence.addPosition(operandNumber, sourcePosition);
                operandStateArray[index] = 1;
            } else if (operandStateArray[index] == 0) {
                operandStateArray[index] = -2;
            }
        }

        @Override
        public LangOccurrence getLangOccurrence(int i) {
            return langOccurrenceList.get(i);
        }

        private InternalLangOccurrence getOrCreateLangOccurrence(Lang lang) {
            int size = langOccurrenceList.size();
            for (int i = 0; i < size; i++) {
                InternalLangOccurrence langOccurrence = langOccurrenceList.get(i);
                if (langOccurrence.getLang().equals(lang)) {
                    return langOccurrence;
                }
            }
            InternalLangOccurrence langOccurrence = new InternalLangOccurrence(lang, operandMaxNumber);
            langOccurrenceList.add(langOccurrence);
            return langOccurrence;
        }

        private float[] checkMotcleScoreArray(int[] acceptOperandArray) {
            int note = 0;
            int operandLength = operandStateArray.length;
            float[] result = new float[operandLength];
            for (int i = 0; i < operandLength; i++) {
                int state = operandStateArray[i];
                if (state == 1) {
                    result[i] = 9;
                } else if (state == -2) {
                    result[i] = -2;
                } else {
                    result[i] = -1;
                }
            }

            int length = acceptOperandArray.length;
            if (length == 0) {
                withAccept = false;
            }
            for (int i = 0; i < length; i++) {
                int operandNumber = acceptOperandArray[i];
                if (operandStateArray[operandNumber - 1] > 0) {
                    note++;
                }
            }
            for (int i = 0; i < length; i++) {
                int operandNumber = acceptOperandArray[i];
                int index = operandNumber - 1;
                if (operandStateArray[index] > 0) {
                    result[index] = ((float) note) / length;
                }
            }
            if (note == 0) {
                withAccept = false;
            } else {
                withAccept = true;
            }
            return result;
        }

        private boolean isWithAccept() {
            return withAccept;
        }

    }


    private static class InternalLangOccurrence implements MotclePertinence.LangOccurrence {

        private final Lang lang;
        private final SubstringPosition[] substringPositionArray;

        private InternalLangOccurrence(Lang lang, int operandMaxNumber) {
            this.lang = lang;
            this.substringPositionArray = new SubstringPosition[operandMaxNumber];
        }

        @Override
        public Lang getLang() {
            return lang;
        }

        @Override
        public SubstringPosition getSubstringPosition(int operandNumber) {
            return substringPositionArray[operandNumber - 1];
        }

        private void addPosition(int operandNumber, SubstringPosition substringPosition) {
            int index = operandNumber - 1;
            SubstringPosition current = substringPositionArray[index];
            if ((current == null) || (current.getBeginIndex() > substringPosition.getBeginIndex())) {
                substringPositionArray[index] = substringPosition;
            }

        }

    }

}
