/* ScrutariLib_SearchEngine - Copyright (c) 2015-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.operands.search;

import java.util.ArrayList;
import java.util.List;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.logicaloperation.LogicalOperationUtils;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.text.TextConstants;
import net.mapeadores.util.text.lexie.LexieParser;
import net.mapeadores.util.text.lexie.LexieParserListener;
import net.mapeadores.util.text.search.SearchToken;
import net.mapeadores.util.text.search.SearchUtils;
import net.mapeadores.util.text.search.SimpleSearchToken;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.searchengine.api.operands.search.AlineaFilter;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.operands.search.SearchTokenOperand;


/**
 *
 * @author Vincent Calame
 */
public final class RawSearchOperationEngine {

    private final MessageHandler messageHandler;
    private final AttributeKeyCoder attributeKeyCoder;
    private final int defaultSearchType;
    private int currentOperandNumber = 1;

    private RawSearchOperationEngine(MessageHandler messageHandler, AttributeKeyCoder attributeKeyCoder, int defaultSearchType) {
        this.messageHandler = messageHandler;
        this.attributeKeyCoder = attributeKeyCoder;
        this.defaultSearchType = defaultSearchType;
    }

    public static SearchOperation parse(List<RawOperation> rawOperationList, MessageHandler messageHandler, AttributeKeyCoder attributeKeyCoder, int defaultSearchType, short mainOperator) {
        RawSearchOperationEngine rawSearchOperationEngine = new RawSearchOperationEngine(messageHandler, attributeKeyCoder, defaultSearchType);
        Operand rootOperand = rawSearchOperationEngine.convert(rawOperationList, mainOperator);
        return rawSearchOperationEngine.toSearchOperation(rootOperand);
    }

    public static SearchOperation parse(RawOperation rawOperation, MessageHandler messageHandler, AttributeKeyCoder attributeKeyCoder, int defaultSearchType) {
        RawSearchOperationEngine rawSearchOperationEngine = new RawSearchOperationEngine(messageHandler, attributeKeyCoder, defaultSearchType);
        Operand rootOperand = rawSearchOperationEngine.convert(rawOperation);
        return rawSearchOperationEngine.toSearchOperation(rootOperand);
    }

    private SearchOperation toSearchOperation(Operand rootOperand) {
        if (rootOperand == null) {
            return null;
        }
        return SearchOperationBuilder.build(rootOperand, (currentOperandNumber - 1));
    }

    private Operand convert(List<RawOperation> rawOperationList, short mainOperator) {
        int size = rawOperationList.size();
        if (size == 1) {
            return convert(rawOperationList.get(0));
        } else {
            List<Operand> resultList = new ArrayList<Operand>();
            for (int i = 0; i < size; i++) {
                Operand operand = convert(rawOperationList.get(i));
                if (operand != null) {
                    resultList.add(operand);
                }
            }
            int resultSize = resultList.size();
            if (resultSize == 0) {
                return null;
            } else if (resultSize == 1) {
                return resultList.get(0);
            } else {
                return LogicalOperationUtils.toSubOperand(resultList.toArray(new Operand[resultSize]), mainOperator);
            }
        }
    }

    private Operand convert(RawOperation rawOperation) {
        QParser parser = new QParser(rawOperation.getQ(), defaultSearchType);
        List<SearchToken> searchTokenList = parser.parse();
        int size = searchTokenList.size();
        if (size == 0) {
            return null;
        }
        AlineaFilter alineaFilter = SearchOperationUtils.parse(rawOperation.getScope(), messageHandler, attributeKeyCoder);
        if (alineaFilter == null) {
            alineaFilter = AlineaFilters.ALL_FILTER;
        }
        Operand rootOperand;
        if (size == 1) {
            rootOperand = SearchOperationUtils.toSearchTokenOperand(currentOperandNumber, true, searchTokenList.get(0), alineaFilter);
            currentOperandNumber++;
        } else {
            SearchTokenOperand[] array = new SearchTokenOperand[size];
            for (int i = 0; i < size; i++) {
                array[i] = SearchOperationUtils.toSearchTokenOperand(currentOperandNumber, true, searchTokenList.get(i), alineaFilter);
                currentOperandNumber++;
            }
            rootOperand = LogicalOperationUtils.toSubOperand(array, rawOperation.getOperator());
        }
        return rootOperand;
    }


    private static class QParser implements LexieParserListener {

        private final String source;
        private final int defaultSearchType;
        private final List<SearchToken> tokenList = new ArrayList<SearchToken>();
        private final List<SimpleSearchToken> quoteList = new ArrayList<SimpleSearchToken>();
        private int currentType = TextConstants.MATCHES;
        private String currentWord = null;
        private boolean onQuote = false;

        private QParser(String source, int defaultSearchType) {
            this.source = source;
            this.defaultSearchType = defaultSearchType;
        }

        private List<SearchToken> parse() {
            LexieParser.parse(source, this);
            if (onQuote) {
                flushQuote();
            } else {
                flushCurrentWord();
            }
            return tokenList;
        }

        @Override
        public void flushLexie(String word, int wordStart) {
            flushCurrentWord();
            currentWord = word;
        }

        @Override
        public void checkSpecialChar(char carac) {
            if (carac == '*') {
                if (currentWord == null) {
                    currentType = TextConstants.ENDSWITH;
                } else {
                    currentType = currentType | TextConstants.STARTSWITH;
                    flushCurrentWord();
                }
            } else if (carac == '\"') {
                if (onQuote) {
                    flushQuote();
                } else {
                    flushCurrentWord();
                    onQuote = true;
                }
            } else {
                flushCurrentWord();
            }
        }

        private void flushCurrentWord() {
            if (currentWord != null) {
                if (onQuote) {
                    quoteList.add(SearchUtils.toSimpleSearchToken(currentWord, currentType));
                } else {
                    tokenList.add(SearchUtils.toSimpleSearchToken(currentWord, currentType | defaultSearchType));
                }
                currentWord = null;
                currentType = TextConstants.MATCHES;
            }
        }

        private void flushQuote() {
            flushCurrentWord();
            int size = quoteList.size();
            if (size > 0) {
                SimpleSearchToken[] array = new SimpleSearchToken[size];
                for (int j = 0; j < size; j++) {
                    array[j] = quoteList.get(j);
                }
                int p = tokenList.size();
                tokenList.add(SearchUtils.toMultiSearchToken(array));
            }
            onQuote = false;
            quoteList.clear();
        }

    }

}
