/* ScrutariLib_SearchEngine - Copyright (c) 2006-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.searchengine.tools.options;

import java.util.ArrayList;
import java.util.Collection;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangList;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.text.LabelUtils;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.api.stats.ScrutariDBStats;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.options.CategoryDef;
import net.scrutari.searchengine.api.options.ScrutariDataURIList;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.tools.EligibilityBuffer;


/**
 *
 * @author Vincent Calame
 */
public final class OptionsUtils {

    public final static ScrutariDataURIList EMPTY_BASELIST = new EmptyScrutariDataURIList(ScrutariDataURI.BASEURI_TYPE);
    public final static ScrutariDataURIList EMPTY_CORPUSLIST = new EmptyScrutariDataURIList(ScrutariDataURI.CORPUSURI_TYPE);
    public final static ScrutariDataURIList EMPTY_THESAURUSLIST = new EmptyScrutariDataURIList(ScrutariDataURI.THESAURUSURI_TYPE);
    public final static ScrutariDataURIList EMPTY_FICHELIST = new EmptyScrutariDataURIList(ScrutariDataURI.FICHEURI_TYPE);
    public final static ScrutariDataURIList EMPTY_MOTCLELIST = new EmptyScrutariDataURIList(ScrutariDataURI.MOTCLEURI_TYPE);

    private OptionsUtils() {
    }

    public static CodeList getSelectionThesaurusCodeList(ScrutariDBStats scrutariDBStats, DataAccess dataAccess, CodeList corpusCodeList) {
        if (corpusCodeList == null) {
            return null;
        }
        Set<Integer> thesaurusSet = new LinkedHashSet<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        CodeList thesaurusCodeList = dataAccess.getThesaurusCodeList();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodeList.getCode(i);
            for (int j = 0; j < thesaurusCount; j++) {
                int codethesaurus = thesaurusCodeList.getCode(j);
                if (scrutariDBStats.hasIndexation(codecorpus, codethesaurus)) {
                    thesaurusSet.add(codethesaurus);
                }
            }
        }
        return CodesUtils.toCodeList(thesaurusSet);
    }

    public static CodeList getFicheCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        SearchOptionsDef searchOptionsDef = searchOptions.getSearchOptionsDef();
        EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptionsDef);
        Set<Lang> langSet = null;
        LangList langList = searchOptionsDef.getFilterLangList();
        int langCount = langList.getLangCount();
        if (langCount > 0) {
            langSet = new HashSet<Lang>();
            for (int i = 0; i < langCount; i++) {
                langSet.add(langList.getLang(i));
            }
        }
        CodeList corpusCodeList = searchOptions.getCorpusCodeList();
        if (corpusCodeList == null) {
            corpusCodeList = dataAccess.getCorpusCodeList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int corpusCount = corpusCodeList.getCodeCount();
        for (int i = 0; i < corpusCount; i++) {
            int codecorpus = corpusCodeList.getCode(i);
            CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
            int ficheCount = corpusData.getFicheCount();
            for (int j = 0; j < ficheCount; j++) {
                int codefiche = corpusData.getCodeFiche(j);
                boolean accept = true;
                if (langSet != null) {
                    Lang lang = dataAccess.getFicheLang(codefiche);
                    if (!langSet.contains(lang)) {
                        accept = false;
                    }
                }
                if ((accept) && (!eligibilityBuffer.acceptFiche(codefiche))) {
                    accept = false;
                }
                if (accept) {
                    list.add(codefiche);
                }
            }
        }
        return CodesUtils.toCodeList(list);
    }

    public static CodeList getMotcleCodeList(DataAccess dataAccess, SearchOptions searchOptions) {
        EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptions.getSearchOptionsDef());
        CodeList thesaurusCodeList = searchOptions.getThesaurusCodeList();
        if (thesaurusCodeList == null) {
            thesaurusCodeList = dataAccess.getThesaurusCodeList();
        }
        List<Integer> list = new ArrayList<Integer>();
        int thesaurusCount = thesaurusCodeList.getCodeCount();
        for (int i = 0; i < thesaurusCount; i++) {
            int codethesaurus = thesaurusCodeList.getCode(i);
            ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(codethesaurus);
            int motcleCount = thesaurusData.getMotcleCount();
            for (int j = 0; j < motcleCount; j++) {
                int codemotcle = thesaurusData.getCodeMotcle(j);
                if (eligibilityBuffer.acceptMotcle(codemotcle)) {
                    list.add(codemotcle);
                }
            }
        }
        return CodesUtils.toCodeList(list);
    }


    public static ScrutariDataURIList toScrutariDataURIList(DataAccess dataAccess, short type, CodeList codeList) {
        List<ScrutariDataURI> list = new ArrayList<ScrutariDataURI>();
        int codeCount = codeList.getCodeCount();
        for (int i = 0; i < codeCount; i++) {
            int code = codeList.getCode(i);
            switch (type) {
                case ScrutariDataURI.BASEURI_TYPE:
                    BaseData baseData = dataAccess.getBaseDataByCode(code);
                    if (baseData != null) {
                        list.add(baseData.getBaseURI());
                    }
                    break;
                case ScrutariDataURI.CORPUSURI_TYPE:
                    CorpusData corpusData = dataAccess.getCorpusDataByCode(code);
                    if (corpusData != null) {
                        list.add(corpusData.getCorpusURI());
                    }
                    break;
                case ScrutariDataURI.THESAURUSURI_TYPE:
                    ThesaurusData thesaurusData = dataAccess.getThesaurusDataByCode(code);
                    if (thesaurusData != null) {
                        list.add(thesaurusData.getThesaurusURI());
                    }
                    break;
                case ScrutariDataURI.FICHEURI_TYPE:
                    FicheData ficheData = dataAccess.getFicheDataByCode(code);
                    if (ficheData != null) {
                        list.add(ficheData.getFicheURI());
                    }
                    break;
                case ScrutariDataURI.MOTCLEURI_TYPE:
                    MotcleData motcleData = dataAccess.getMotcleDataByCode(code);
                    if (motcleData != null) {
                        list.add(motcleData.getMotcleURI());
                    }
                    break;
                default:
                    throw new SwitchException("Unkown type: " + type);
            }
        }
        ScrutariDataURI[] array = list.toArray(new ScrutariDataURI[list.size()]);
        return new ArrayScrutariDataURIList(type, array);
    }

    public static ScrutariDataURIList toScrutariDataURIList(short type, Collection<ScrutariDataURI> collection) {
        ScrutariDataURI[] array = collection.toArray(new ScrutariDataURI[collection.size()]);
        return new ArrayScrutariDataURIList(type, array);
    }

    public static ScrutariDataURIList getEmptyScrutariDataURIList(short type) {
        switch (type) {
            case ScrutariDataURI.BASEURI_TYPE:
                return EMPTY_BASELIST;
            case ScrutariDataURI.CORPUSURI_TYPE:
                return EMPTY_CORPUSLIST;
            case ScrutariDataURI.THESAURUSURI_TYPE:
                return EMPTY_THESAURUSLIST;
            case ScrutariDataURI.FICHEURI_TYPE:
                return EMPTY_FICHELIST;
            case ScrutariDataURI.MOTCLEURI_TYPE:
                return EMPTY_MOTCLELIST;
            default:
                throw new SwitchException("Unkown type: " + type);
        }
    }

    public static String checkLibelleString(CategoryDef category, Lang lang, String defaultString) {
        Label libelle = LabelUtils.getLangPartCheckedLabel(category, lang);
        if (libelle == null) {
            libelle = LabelUtils.getFirstLabel(category);
        }
        if (libelle != null) {
            return libelle.getLabelString();
        } else {
            return defaultString;
        }
    }


    private static class ArrayScrutariDataURIList implements ScrutariDataURIList {

        private final short type;
        private final ScrutariDataURI[] array;

        private ArrayScrutariDataURIList(short type, ScrutariDataURI[] array) {
            this.type = type;
            this.array = array;
        }

        @Override
        public short getType() {
            return type;
        }

        @Override
        public int getScrutariDataURICount() {
            return array.length;
        }

        @Override
        public ScrutariDataURI getScrutariDataURI(int index) {
            return array[index];
        }

    }


    private static class EmptyScrutariDataURIList implements ScrutariDataURIList {

        private final short type;

        private EmptyScrutariDataURIList(short type) {
            this.type = type;
        }

        @Override
        public short getType() {
            return type;
        }

        @Override
        public int getScrutariDataURICount() {
            return 0;
        }

        @Override
        public ScrutariDataURI getScrutariDataURI(int index) {
            throw new IndexOutOfBoundsException();
        }

    }

}
