/* ScrutariLib_Xml - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml;

import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LabelElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.xml.xmlpull.CorpusHandler;
import net.scrutari.xml.xmlpull.ThesaurusHandler;
import net.scrutari.xml.xmlpull.XmlPullUtils;
import org.xmlpull.v1.XmlPullParser;
import org.xmlpull.v1.XmlPullParserException;
import org.xmlpull.v1.XmlPullParserFactory;


/**
 *
 * @author Vincent Calame
 */
public class BaseXmlEngine {

    private final XmlPullParserFactory factory;
    public static final int BASE_ICON = 1;
    public static final int LANG = 2;

    public BaseXmlEngine() {
        try {
            this.factory = XmlPullParserFactory.newInstance(System.getProperty(XmlPullParserFactory.PROPERTY_NAME), null);
        } catch (XmlPullParserException xppe) {
            throw new NestedLibraryException(xppe);
        }
    }

    public void run(String scrutariSourceName, BaseData.Builder baseDataBuilder, Parameters parameters, InputStream inputStream, boolean buildBaseMetadata) throws XmlPullParserException, IOException {
        XmlPullParser xpp = factory.newPullParser();
        xpp.setInput(inputStream, null);
        RootHandler rootHandler = null;
        int eventType = xpp.getEventType();
        do {
            if (eventType == XmlPullParser.START_TAG) {
                String tagName = xpp.getName();
                if (!IndexationXmlEngine.isElement(tagName)) {
                    if (rootHandler == null) {
                        if (tagName.equals("base")) {
                            rootHandler = new RootHandler(baseDataBuilder, parameters, buildBaseMetadata);
                        } else {
                            parameters.getMessageHandler().addMessage(ScrutariXmlConstants.SEVERE_CATEGORY, LocalisationUtils.toMessage("_ error.wrong.xml.root", LogUtils.toValue(tagName, "base")));
                            break;
                        }
                    } else {
                        Map<String, String> map = null;
                        int count = xpp.getAttributeCount();
                        if (count > 0) {
                            map = new HashMap<String, String>();
                            for (int i = 0; i < count; i++) {
                                map.put(xpp.getAttributeName(i), xpp.getAttributeValue(i));
                            }
                        }
                        rootHandler.processStartElement(xpp.getName(), map);
                    }
                }
            } else if (eventType == XmlPullParser.END_TAG) {
                String tagName = xpp.getName();
                if (!IndexationXmlEngine.isElement(tagName)) {
                    rootHandler.processEndElement(tagName);
                }
            } else if (eventType == XmlPullParser.TEXT) {
                if (rootHandler != null) {
                    rootHandler.processText(xpp.getText());
                }
            }
            eventType = xpp.next();
        } while (eventType != XmlPullParser.END_DOCUMENT);
    }


    private static class RootHandler extends DefaultElementHandler {

        private final BaseData.Builder baseDataBuilder;
        private final Parameters parameters;
        private final MessageHandler messageHandler;
        private final boolean buildBaseMetadata;

        private RootHandler(BaseData.Builder baseDataBuilder, Parameters parameters, boolean buildBaseMetadata) {
            this.baseDataBuilder = baseDataBuilder;
            this.parameters = parameters;
            this.messageHandler = parameters.getMessageHandler();
            this.buildBaseMetadata = buildBaseMetadata;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            switch (tagname) {
                case "base-metadata":
                    if (buildBaseMetadata) {
                        return new MetadataHandler(baseDataBuilder, parameters.getMessageHandler());
                    } else {
                        return null;
                    }
                case "corpus":
                    String corpusName = getAttributeString(attributeMap, "corpus-name");
                    if (corpusName == null) {
                        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage("_ error.empty.xml.attribute", LogUtils.toValue("/base/corpus", "corpus-name")));
                        return null;
                    }
                    try {
                        return new CorpusHandler(corpusName, parameters);
                    } catch (IllegalArgumentException iae) {
                        return null;
                    }
                case "thesaurus":
                    String thesaurusName = getAttributeString(attributeMap, "thesaurus-name");
                    if (thesaurusName == null) {
                        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage("_ error.empty.xml.attribute", LogUtils.toValue("/base/thesaurus", "thesaurus-name")));
                        return null;
                    }
                    try {
                        return new ThesaurusHandler(thesaurusName, parameters);
                    } catch (IllegalArgumentException iae) {
                        return null;
                    }
                default:
                    return null;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof CorpusHandler) {
                CorpusData corpusData = ((CorpusHandler) elementHandler).flushCorpusData();
                if (corpusData != null) {
                    baseDataBuilder.addCodeCorpus(corpusData.getCodeCorpus());
                }
            } else if (elementHandler instanceof ThesaurusHandler) {
                ThesaurusData thesaurusData = ((ThesaurusHandler) elementHandler).flushThesaurusData();
                if (thesaurusData != null) {
                    baseDataBuilder.addCodeThesaurus(thesaurusData.getCodeThesaurus());
                }
            }
        }

    }


    private static class MetadataHandler extends DefaultElementHandler {

        private final BaseData.Builder baseDataBuilder;
        private final MessageHandler messageHandler;


        private MetadataHandler(BaseData.Builder baseDataBuilder, MessageHandler messageHandler) {
            this.baseDataBuilder = baseDataBuilder;
            this.messageHandler = messageHandler;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("base-icon")) {
                return new SimpleElementHandler(BASE_ICON);
            } else if (tagname.equals("intitule-short")) {
                return new IntituleHandler(baseDataBuilder, messageHandler, BaseData.INTITULE_SHORT, "/base/base-metadata/intitule-short");
            } else if (tagname.equals("intitule-long")) {
                return new IntituleHandler(baseDataBuilder, messageHandler, BaseData.INTITULE_LONG, "/base/base-metadata/intitule-long");
            } else if (tagname.equals("langs-ui")) {
                return new LangUiHandler(baseDataBuilder, messageHandler, "/base/base-metadata/langs-ui");
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            if (text.length() == 0) {
                text = null;
            }
            switch (type) {
                case BASE_ICON:
                    baseDataBuilder.setBaseIcon(text);
                    break;
            }
        }

    }


    private static class IntituleHandler extends DefaultElementHandler {

        private final BaseData.Builder baseDataBuilder;
        private final MessageHandler messageHandler;
        private final short type;
        private final String xpath;

        private IntituleHandler(BaseData.Builder baseDataBuilder, MessageHandler messageHandler, short type, String xpath) {
            this.baseDataBuilder = baseDataBuilder;
            this.messageHandler = messageHandler;
            this.type = type;
            this.xpath = xpath;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LabelElementHandler(attributeMap, xpath + "/" + tagname);
                } catch (ErrorMessageException eme) {
                    messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage(eme.getErrorMessage().getMessageKey(), eme.getErrorMessage().getMessageValue()));
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Label libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                baseDataBuilder.addLibelle(type, libelle);
            }

        }

    }


    private static class LangUiHandler extends DefaultElementHandler {

        private final BaseData.Builder baseDataBuilder;
        private final MessageHandler messageHandler;
        private final String xpath;

        private LangUiHandler(BaseData.Builder baseDataBuilder, MessageHandler messageHandler, String xpath) {
            this.baseDataBuilder = baseDataBuilder;
            this.messageHandler = messageHandler;
            this.xpath = xpath;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map attributeMap) {
            if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            try {
                Lang lang = Lang.parse(text);
                baseDataBuilder.addLangUI(lang);
            } catch (ParseException ile) {
                messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage("_ error.wrong.xml.tagvalue", LogUtils.toValue(xpath + "/lang", text)));
            }
        }

    }

}
