/* ScrutariLib_Xml - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml.xmlpull;

import java.text.ParseException;
import java.util.ArrayList;
import java.util.List;
import java.util.Map;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LabelElementHandler;
import net.mapeadores.util.xml.xmlpull.SimpleElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.data.ScrutariAttributes;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.xml.Parameters;
import net.scrutari.xml.ScrutariXmlConstants;


/**
 *
 * @author Vincent Calame
 */
public class CorpusHandler extends DefaultElementHandler {

    private static final int HREF_PARENT = 1;
    private static final int TITRE = 3;
    private static final int SOUSTITRE = 4;
    private static final int DATE = 5;
    private static final int LANG = 6;
    private static final int HREF = 7;
    private static final int COMPLEMENT = 8;
    private static final int FICHE_ICON = 9;
    private static final int LATITUDE = 10;
    private static final int LONGITUDE = 11;
    private static final int VALUE = 12;
    private final DataCacheWriter dataCacheWriter;
    private final DataCoder dataCoder;
    private final MessageHandler messageHandler;
    private final CorpusURI corpusURI;
    private final int codebase;
    private final int codecorpus;
    private final CorpusData.Builder corpusDataBuilder;
    private final FicheHandler ficheHandler = new FicheHandler();
    private final String corpusxpath;
    private boolean metadataDone = false;

    public CorpusHandler(String corpusName, Parameters parameters) {
        this.messageHandler = parameters.getMessageHandler();
        this.corpusxpath = "/base/corpus[@corpus-name='" + corpusName + "']";
        this.dataCacheWriter = parameters.getDataCacheWriter();
        this.dataCoder = parameters.getDataCoder();
        corpusURI = new CorpusURI(parameters.getBaseURI(), corpusName);
        if (dataCoder.checkCode(corpusURI) != -1) {
            addWarning("_ error.existing.xml.attributevalue", LogUtils.toValue(corpusxpath, "corpus-name", corpusName));
            throw new IllegalArgumentException();
        }
        this.codecorpus = dataCoder.getCode(corpusURI);
        this.codebase = parameters.getCodeBase();
        corpusDataBuilder = new CorpusData.Builder(corpusURI);
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("corpus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                addWarning("_ error.unsupported.xml.missingtag", LogUtils.toValue(corpusxpath, "corpus-metadata"));
                return null;
            }
        } else {
            if (tagname.equals("fiche")) {
                String ficheId = getAttributeString(attributeMap, "fiche-id");
                if (ficheId == null) {
                    addWarning("_ error.empty.xml.attribute", LogUtils.toValue(corpusxpath + "/fiche", "fiche-id"));
                    return null;
                }
                String ficheXpath = corpusxpath + "/fiche[@fiche-id='" + ficheId + "']";
                FicheURI ficheURI = new FicheURI(corpusURI, ficheId);
                if (dataCoder.checkCode(ficheURI) != -1) {
                    addWarning("_ error.existing.xml.attributevalue", LogUtils.toValue(ficheXpath, "fiche-id", ficheId));
                    return null;
                }
                int codefiche = dataCoder.getCode(ficheURI);
                ficheHandler.newFiche(ficheURI, codefiche, ficheXpath);
                return ficheHandler;
            } else {
                return null;
            }
        }
    }

    @Override
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof FicheHandler) {
            FicheHandler fh = (FicheHandler) elementHandler;
            FicheData.Builder ficheDataBuilder = fh.getFicheDataBuilder();
            int codefiche = fh.getCodeFiche();
            FicheData ficheData = ficheDataBuilder.toFicheData(codefiche, corpusDataBuilder.getHrefParent(), codecorpus, codebase);
            dataCacheWriter.cacheFicheData(ficheData);
            corpusDataBuilder.addCodeFiche(codefiche);
        }
    }

    public CorpusData flushCorpusData() {
        if (!metadataDone) {
            return null;
        }
        CorpusData corpusData = corpusDataBuilder.toCorpusData(codecorpus, codebase);
        dataCacheWriter.cacheCorpusData(corpusData);
        return corpusData;
    }

    private void addWarning(String key, Object value) {
        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage(key, value));
    }


    private class MetadataHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("intitule-corpus")) {
                return new IntituleHandler(corpusxpath + "/corpus-metadata/intitule-corpus", CorpusData.INTITULECORPUS_FIELDNUMBER);
            } else if (tagname.equals("intitule-fiche")) {
                return new IntituleHandler(corpusxpath + "/corpus-metadata/intitule-fiche", CorpusData.INTITULEFICHE_FIELDNUMBER);
            } else if (tagname.equals("complement-metadata")) {
                int fieldNumber = corpusDataBuilder.createNewComplement();
                return new IntituleHandler(corpusxpath + "/corpus-metadata/complement-metadata", fieldNumber);
            } else if (tagname.equals("href-parent")) {
                return new SimpleElementHandler(HREF_PARENT);
            } else if (tagname.equals("corpus-icon")) {
                return null;//return new SimpleElementHandler(TYPE);
            } else {
                return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case HREF_PARENT:
                    corpusDataBuilder.setHrefParent(text);
                    break;
            }
        }

    }


    private class FicheHandler extends DefaultElementHandler {

        private String ficheXpath;
        private FicheURI ficheURI;
        private int codefiche;
        private FicheData.Builder ficheDataBuilder;

        private FicheHandler() {
        }

        private void newFiche(FicheURI ficheURI, int codefiche, String ficheXpath) {
            this.ficheURI = ficheURI;
            this.codefiche = codefiche;
            this.ficheXpath = ficheXpath;
            ficheDataBuilder = new FicheData.Builder(ficheURI, corpusDataBuilder.getComplementMaxNumber());
        }

        private FicheData.Builder getFicheDataBuilder() {
            return ficheDataBuilder;
        }

        private int getCodeFiche() {
            return codefiche;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("titre")) {
                return new SimpleElementHandler(TITRE);
            } else if (tagname.equals("soustitre")) {
                return new SimpleElementHandler(SOUSTITRE);
            } else if (tagname.equals("date")) {
                return new SimpleElementHandler(DATE);
            } else if (tagname.equals("lang")) {
                return new SimpleElementHandler(LANG);
            } else if (tagname.equals("href")) {
                return new SimpleElementHandler(HREF);
            } else if (tagname.equals("complement")) {
                return new SimpleElementHandler(COMPLEMENT);
            } else if (tagname.equals("fiche-icon")) {
                return new SimpleElementHandler(FICHE_ICON);
            } else if (tagname.equals("geoloc")) {
                return new GeolocHandler();
            } else if (tagname.equals("attr")) {
                String localKey = attributeMap.get("key");
                if (localKey != null) {
                    localKey = localKey.trim();
                }
                if ((localKey == null) || (localKey.isEmpty())) {
                    addWarning("_ error.empty.xml.attribute", LogUtils.toValue(ficheXpath + "/attr", "key"));
                    return null;
                }
                String nameSpace = attributeMap.get("ns");
                if (nameSpace != null) {
                    nameSpace = nameSpace.trim();
                }
                if ((nameSpace == null) || (nameSpace.isEmpty())) {
                    addWarning("_ error.empty.xml.attribute", LogUtils.toValue(ficheXpath + "/attr[@key='" + localKey + "']", "ns"));
                    return null;
                }
                try {
                    AttributeKey attributeKey = ScrutariAttributes.toAttributeKey(nameSpace, localKey);
                    return new AttrHandler(attributeKey);
                } catch (ParseException pe) {
                    addWarning("_ error.wrong.xml.attributevalue", LogUtils.toValue(ficheXpath + "/attr[@key='" + localKey + "' AND ns='" + nameSpace + "']", "key|ns", nameSpace + ":" + localKey));
                    return null;
                }
            } else {
                return null;
            }
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case TITRE:
                    ficheDataBuilder.setTitre(text);
                    break;
                case SOUSTITRE:
                    ficheDataBuilder.setSoustitre(text);
                    break;
                case DATE:
                    try {
                        if ((text != null) && (text.length() != 0)) {
                            ficheDataBuilder.setDateInteger(DateInteger.parse(text));
                        }
                    } catch (DateIntegerException ide) {
                        addWarning("_ error.wrong.xml.tagvalue", LogUtils.toValue(ficheXpath + "/date", text));
                    }
                    break;
                case LANG:
                    try {
                        if ((text != null) && (text.length() != 0)) {
                            ficheDataBuilder.setLang(Lang.parse(text));
                        }
                    } catch (ParseException ile) {
                        addWarning("_ error.wrong.xml.tagvalue", LogUtils.toValue(ficheXpath + "/lang", text));
                    }
                    break;
                case HREF:
                    ficheDataBuilder.setHref(text);
                    break;
                case COMPLEMENT:
                    boolean done = ficheDataBuilder.addComplement(text);
                    if (!done) {
                        addWarning("_ error.unsupported.xml.toomanytags", ficheXpath + "/complement");
                    }
                    break;
                case FICHE_ICON:
                    ficheDataBuilder.setFicheIcon(text);
                    break;
            }
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            if (elementHandler instanceof GeolocHandler) {
                GeolocHandler geolocHandler = (GeolocHandler) elementHandler;
                String latitude = geolocHandler.latitude;
                String longitude = geolocHandler.longitude;
                boolean done = true;
                if (latitude == null) {
                    done = false;
                    addWarning("_ error.unsupported.xml.missingtag", LogUtils.toValue(ficheXpath + "/geoloc", "lat"));
                }
                if (longitude == null) {
                    done = false;
                    addWarning("_ error.unsupported.xml.missingtag", LogUtils.toValue(ficheXpath + "/geoloc", "lon"));
                }
                if (done) {
                    ficheDataBuilder.setGeoloc(latitude, longitude);
                }
            } else if (elementHandler instanceof AttrHandler) {
                AttrHandler attrHandler = (AttrHandler) elementHandler;
                ficheDataBuilder.addAttribute(attrHandler.attributeKey, attrHandler.valueList);
            }
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        private final String xpath;
        private final int fieldNumber;

        IntituleHandler(String xpath, int fieldNumber) {
            this.xpath = xpath;
            this.fieldNumber = fieldNumber;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LabelElementHandler(attributeMap, xpath + "/lib");
                } catch (ErrorMessageException eme) {
                    addWarning(eme.getErrorMessage().getMessageKey(), eme.getErrorMessage().getMessageValue());
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Label libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                corpusDataBuilder.addLibelle(fieldNumber, libelle);
            }
        }

    }


    private class GeolocHandler extends DefaultElementHandler {

        private String latitude;
        private String longitude;

        GeolocHandler() {
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lat")) {
                return new SimpleElementHandler(LATITUDE);
            } else if (tagname.equals("lon")) {
                return new SimpleElementHandler(LONGITUDE);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case LATITUDE:
                    latitude = text;
                    break;
                case LONGITUDE:
                    longitude = text;
                    break;
            }
        }

    }


    private class AttrHandler extends DefaultElementHandler {

        private AttributeKey attributeKey;
        private List<CleanedString> valueList = new ArrayList<CleanedString>();

        private AttrHandler(AttributeKey attributeKey) {
            this.attributeKey = attributeKey;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("val")) {
                return new SimpleElementHandler(VALUE);
            }
            return null;
        }

        @Override
        public void endSimpleSubElement(int type, String text) {
            switch (type) {
                case VALUE:
                    CleanedString cs = CleanedString.newInstance(text);
                    if (cs != null) {
                        valueList.add(cs);
                    }
                    break;
            }
        }

    }

}
