/* ScrutariLib_Xml - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.scrutari.xml.xmlpull;

import java.util.Map;
import net.mapeadores.util.localisation.LocalisationUtils;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.text.Label;
import net.mapeadores.util.xml.xmlpull.DefaultElementHandler;
import net.mapeadores.util.xml.xmlpull.LabelElementHandler;
import net.mapeadores.util.xml.xmlpull.XmlPullElementHandler;
import net.scrutari.data.MotcleData;
import net.scrutari.data.ThesaurusData;
import net.scrutari.datauri.MotcleURI;
import net.scrutari.datauri.ThesaurusURI;
import net.scrutari.db.api.DataCoder;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.xml.Parameters;
import net.scrutari.xml.ScrutariXmlConstants;


/**
 *
 * @author Vincent Calame
 */
public class ThesaurusHandler extends DefaultElementHandler {

    private final DataCacheWriter dataCacheWriter;
    private final DataCoder dataCoder;
    private final MessageHandler messageHandler;
    private final ThesaurusURI thesaurusURI;
    private final int codebase;
    private final int codethesaurus;
    private final ThesaurusData.Builder thesaurusDataBuilder;
    private final MotcleHandler motcleHandler = new MotcleHandler();
    private final String thesaurusxpath;
    private boolean metadataDone = false;

    public ThesaurusHandler(String thesaurusName, Parameters parameters) {
        this.messageHandler = parameters.getMessageHandler();
        this.thesaurusxpath = "/base/thesaurus[@thesaurus-name='" + thesaurusName + "']";
        this.dataCacheWriter = parameters.getDataCacheWriter();
        this.dataCoder = parameters.getDataCoder();
        this.thesaurusURI = new ThesaurusURI(parameters.getBaseURI(), thesaurusName);
        if (dataCoder.checkCode(thesaurusURI) != -1) {
            addWarning("_ error.existing.xml.attributevalue", LogUtils.toValue(thesaurusxpath, "thesaurus-name", thesaurusName));
            throw new IllegalArgumentException();
        }
        this.codethesaurus = dataCoder.getCode(thesaurusURI);
        this.codebase = parameters.getCodeBase();
        thesaurusDataBuilder = new ThesaurusData.Builder(thesaurusURI);
    }

    public ThesaurusData flushThesaurusData() {
        if (!metadataDone) {
            return null;
        }
        ThesaurusData thesaurusData = thesaurusDataBuilder.toThesaurusData(codethesaurus, codebase);
        dataCacheWriter.cacheThesaurusData(thesaurusData);
        return thesaurusData;
    }

    @Override
    public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
        if (!metadataDone) {
            if (tagname.equals("thesaurus-metadata")) {
                metadataDone = true;
                return new MetadataHandler();
            } else {
                addWarning("_ error.unsupported.xml.missingtag", LogUtils.toValue(thesaurusxpath, "thesaurus-metadata"));
                return null;
            }
        } else {
            if (tagname.equals("motcle")) {
                String motcleId = getAttributeString(attributeMap, "motcle-id");
                if (motcleId == null) {
                    addWarning("_ error.empty.xml.attribute", LogUtils.toValue(thesaurusxpath + "/motcle", "motcle-id"));
                    return null;
                }
                MotcleURI motcleURI = new MotcleURI(thesaurusURI, motcleId);
                if (dataCoder.checkCode(motcleURI) != -1) {
                    addWarning("_ error.existing.xml.attributevalue", LogUtils.toValue(thesaurusxpath + "/motcle[motcle-id='" + motcleId + "']", "motcle-id", motcleId));
                    return null;
                }
                int codemotcle = dataCoder.getCode(motcleURI);
                motcleHandler.newMotcle(motcleURI, codemotcle);
                return motcleHandler;
            } else {
                return null;
            }
        }
    }

    @Override
    public void endSubElement(XmlPullElementHandler elementHandler) {
        if (elementHandler instanceof MotcleHandler) {
            MotcleHandler mh = (MotcleHandler) elementHandler;
            MotcleData.Builder motcleDataBuilder = mh.getMotcleDataBuilder();
            int codemotcle = mh.getCodeMotcle();
            MotcleData motcleData = motcleDataBuilder.toMotcleData(codemotcle, codethesaurus, codebase);
            dataCacheWriter.cacheMotcleData(motcleData);
            thesaurusDataBuilder.addCodeMotcle(codemotcle);
        }
    }

    private void addWarning(String key, Object value) {
        messageHandler.addMessage(ScrutariXmlConstants.WARNING_CATEGORY, LocalisationUtils.toMessage(key, value));
    }


    private class MetadataHandler extends DefaultElementHandler {

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("intitule-thesaurus")) {
                return new IntituleHandler(thesaurusxpath + "/thesaurus-metadata/intitule-thesaurus");
            } else {
                return null;
            }
        }

    }


    private class IntituleHandler extends DefaultElementHandler {

        private final String xpath;

        private IntituleHandler(String xpath) {
            this.xpath = xpath;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LabelElementHandler(attributeMap, xpath + "/lib");
                } catch (ErrorMessageException eme) {
                    addWarning(eme.getErrorMessage().getMessageKey(), eme.getErrorMessage().getMessageValue());
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Label libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                thesaurusDataBuilder.addLibelle(libelle);
            }
        }

    }


    private class MotcleHandler extends DefaultElementHandler {

        private MotcleURI motcleURI;
        private int codemotcle;
        private MotcleData.Builder motcleDataBuilder;

        private MotcleHandler() {
        }

        private void newMotcle(MotcleURI motcleURI, int codemotcle) {
            this.motcleURI = motcleURI;
            this.codemotcle = codemotcle;
            motcleDataBuilder = new MotcleData.Builder(motcleURI);
        }

        private MotcleData.Builder getMotcleDataBuilder() {
            return motcleDataBuilder;
        }

        private int getCodeMotcle() {
            return codemotcle;
        }

        @Override
        public XmlPullElementHandler startSubElement(String tagname, Map<String, String> attributeMap) {
            if (tagname.equals("lib")) {
                try {
                    return new LabelElementHandler(attributeMap, thesaurusxpath + "/motcle[motcle-id='" + motcleURI.getMotcleId() + "']/lib");
                } catch (ErrorMessageException eme) {
                    addWarning(eme.getErrorMessage().getMessageKey(), eme.getErrorMessage().getMessageValue());
                    return null;
                }
            }
            return null;
        }

        @Override
        public void endSubElement(XmlPullElementHandler elementHandler) {
            Label libelle = XmlPullUtils.testLibelle(elementHandler);
            if (libelle != null) {
                motcleDataBuilder.addLibelle(libelle);
            }
        }

    }

}
