/* SctServer - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.impl;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.SctServer;
import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.feed.SctFeedCache;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.tools.AddURITreeProvider;
import fr.exemole.sctserver.tools.BuildingUtils;
import fr.exemole.sctserver.tools.feed.SctFeedInfoBuilder;
import net.mapeadores.util.exceptions.ShouldNotOccurException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.TimeLog;
import net.mapeadores.util.logicaloperation.Operand;
import net.scrutari.datauri.ScrutariDataURIChecker;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.searchengine.FicheSearchEngine;
import net.scrutari.searchengine.api.CanonicalQ;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.operands.filters.UnknownOperandTypeException;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchSource;
import net.scrutari.searchengine.tools.OperandCheck;
import net.scrutari.searchengine.tools.operands.filters.FilterOperandConverterEngine;
import net.scrutari.searchengine.tools.operands.filters.OperandConverterParameters;
import net.scrutari.searchengine.tools.operands.search.LogicalSearchOperationEngine;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;
import net.scrutari.supermotcle.api.SupermotcleProvider;
import net.scrutari.supermotcle.tools.SupermotcleProviderBuilder;


/**
 *
 * @author Vincent Calame
 */
class ScrutariSessionImpl implements ScrutariSession {

    private final SctServer sctServer;
    private final ScrutariDB scrutariDB;
    private final ScrutariDataURIChecker scrutariDataURIChecker;
    private final SupermotcleProvider supermotcleProvider;
    private final SctFeedInfo sctFeedInfo;
    private final SctFeedCache sctFeedCache;
    private GlobalSearchOptionsImpl globalSearchOptions;
    private int subid;

    private ScrutariSessionImpl(SctServer sctServer, ScrutariDB scrutariDB, ScrutariDataURIChecker scrutariDataURIChecker, SupermotcleProvider supermotcleProvider) {
        this.sctServer = sctServer;
        this.scrutariDB = scrutariDB;
        this.scrutariDataURIChecker = scrutariDataURIChecker;
        this.supermotcleProvider = supermotcleProvider;
        ScrutariDBName scrutariDBName = scrutariDB.getScrutariDBName();
        this.sctFeedInfo = SctFeedInfoBuilder.build(sctServer, scrutariDBName);
        SctServerStorage sctServerStorage = sctServer.getSctServerStorage();
        this.sctFeedCache = sctServerStorage.getSctFeedCache(scrutariDBName);
        this.subid = sctServerStorage.getQIdStorage().getMaxSubId(scrutariDBName) + 1;
    }

    @Override
    public String getSctServerName() {
        return sctServer.getName();
    }

    @Override
    public SctServer getSctServer() {
        return sctServer;
    }

    @Override
    public ScrutariDB getScrutariDB() {
        return scrutariDB;
    }

    @Override
    public ScrutariDataURIChecker getScrutariDataURIChecker() {
        return scrutariDataURIChecker;
    }

    @Override
    public SupermotcleProvider getSupermotcleProvider() {
        return supermotcleProvider;
    }

    @Override
    public SctFeedInfo getSctFeedInfo() {
        return sctFeedInfo;
    }

    @Override
    public SctFeedCache getSctFeedCache() {
        return sctFeedCache;
    }

    @Override
    public GlobalSearchOptions getGlobalSearchOptions() {
        return globalSearchOptions;
    }

    @Override
    public FicheSearchResult search(SearchOperation searchOperation, SearchOptions searchOptions) {
        SctServerStorage sctServerStorage = sctServer.getSctServerStorage();
        QId existingQId = sctServerStorage.getQIdStorage().containsQId(searchOperation.getCanonicalQ(), searchOptions.getSearchOptionsDef());
        if (existingQId != null) {
            FicheSearchResult existingFicheSearchResult = getSearch(existingQId);
            if (existingFicheSearchResult != null) {
                return existingFicheSearchResult;
            }
        }
        ScrutariDBName scrutariDBName = scrutariDB.getScrutariDBName();
        QId qId;
        synchronized (this) {
            qId = new QId(scrutariDBName, subid);
            subid++;
        }
        DataAccess dataAccess = scrutariDB.openDataAccess();
        FicheSearchResult ficheSearchResult = FicheSearchEngine.search(scrutariDB, searchOperation, searchOptions, getGlobalSearchOptions(), qId, true);
        sctServerStorage.getFicheSearchCache(scrutariDBName).saveFicheSearchResult(ficheSearchResult);
        sctServerStorage.getQIdStorage().saveFicheSearchSource(ficheSearchResult.getFicheSearchSource());
        dataAccess.close();
        return ficheSearchResult;
    }

    @Override
    public FicheSearchResult getSearch(QId qId) {
        SctServerStorage sctServerStorage = sctServer.getSctServerStorage();
        FicheSearchResult ficheSearchResult = sctServerStorage.getFicheSearchCache(scrutariDB.getScrutariDBName()).getFicheSearchResult(qId);
        if (ficheSearchResult != null) {
            return ficheSearchResult;
        }
        FicheSearchSource ficheSearchSource = sctServerStorage.getQIdStorage().getFicheSearchSource(qId);
        if (ficheSearchSource == null) {
            return null;
        }
        SearchOptionsDef searchOptionsDef = ficheSearchSource.getSearchOptionsDef();
        SearchOptions searchOptions;
        DataAccess dataAccess = scrutariDB.openDataAccess();
        if (searchOptionsDef != null) {
            SearchOptionsDefBuilder searchOptionsDefBuilder = SearchOptionsDefBuilder.derive(searchOptionsDef, false);
            SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder(searchOptionsDefBuilder);
            BuildingUtils.computeReduction(searchOptionsBuilder, dataAccess, globalSearchOptions);
            OperandConverterParameters operandConverterParameters = new OperandConverterParameters();
            operandConverterParameters.setDataAccess(dataAccess);
            operandConverterParameters.setFicheSearchResultHolder(this);
            operandConverterParameters.setMessageHandler(LogUtils.NULL_MULTIMESSAGEHANDLER);
            operandConverterParameters.setUriTreeProvider(new AddURITreeProvider(sctServerStorage));
            Operand ficheEligibilityOperand = searchOptionsDef.getFicheEligibilityOperand();
            if (ficheEligibilityOperand != null) {
                try {
                    OperandCheck operandCheck = FilterOperandConverterEngine.convert(ficheEligibilityOperand, searchOptionsBuilder.getCurrentCorpusList(), null, operandConverterParameters);
                    if (!operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
                        searchOptionsBuilder.addFicheOperandCheck(operandCheck);
                    }
                } catch (UnknownOperandTypeException uote) {
                    throw new ShouldNotOccurException(uote);
                }
            }
            Operand motcleEligibilityOperand = searchOptionsDef.getMotcleEligibilityOperand();
            if (motcleEligibilityOperand != null) {
                try {
                    OperandCheck operandCheck = FilterOperandConverterEngine.convert(ficheEligibilityOperand, searchOptionsBuilder.getCurrentCorpusList(), null, operandConverterParameters);
                    if (!operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
                        searchOptionsBuilder.addMotcleOperandCheck(operandCheck);
                    }
                } catch (UnknownOperandTypeException uote) {
                    throw new ShouldNotOccurException(uote);
                }
            }
            searchOptions = searchOptionsBuilder.toSearchOptions();
        } else {
            searchOptions = SearchOptionsBuilder.EMPTY_SEARCHOPTIONS;
        }
        CanonicalQ canonicalQ = ficheSearchSource.getCanonicalQ();
        SearchOperation searchOperation = LogicalSearchOperationEngine.parse(canonicalQ.toString(), LogUtils.NULL_MULTIMESSAGEHANDLER, scrutariDB.getAttributeKeyCoder());
        if (searchOperation == null) {
            dataAccess.close();
            return null;
        }
        ficheSearchResult = FicheSearchEngine.search(scrutariDB, searchOperation, searchOptions, getGlobalSearchOptions(), qId, false);
        sctServerStorage.getFicheSearchCache(scrutariDB.getScrutariDBName()).saveFicheSearchResult(ficheSearchResult);
        dataAccess.close();
        return ficheSearchResult;
    }

    void reloadConf(ConfDirectory confDirectory) {
        globalSearchOptions = GlobalSearchOptionsImpl.newInstance(scrutariDB, confDirectory);
        sctServer.getSctServerStorage().cleanTransientCache(scrutariDB.getScrutariDBName());
    }

    static ScrutariSessionImpl newInstance(SctServer sctServer, ScrutariDB scrutariDB, ScrutariDataURIChecker scrutariDataURIChecker, ConfDirectory confDirectory, TimeLog timeLog) {
        SupermotcleProvider supermotcleProvider = SupermotcleProviderBuilder.build(scrutariDB);
        timeLog.log("scrutarisession-supermotcle_build");
        ScrutariSessionImpl scrutariSession = new ScrutariSessionImpl(sctServer, scrutariDB, scrutariDataURIChecker, supermotcleProvider);
        scrutariSession.reloadConf(confDirectory);
        timeLog.log("scrutarisession-globalsearchoptions_loaded");
        return scrutariSession;
    }

}
