/* SctServer - Copyright (c) 2010-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.request;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.request.json.StartParameters;
import fr.exemole.sctserver.tools.AddURITreeProvider;
import fr.exemole.sctserver.tools.BuildingUtils;
import java.text.ParseException;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashSet;
import java.util.LinkedHashSet;
import java.util.List;
import java.util.Set;
import net.mapeadores.util.attr.AttributeKey;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.localisation.LangListUtils;
import net.mapeadores.util.localisation.Message;
import net.mapeadores.util.localisation.MessageListBuilder;
import net.mapeadores.util.logging.CommandMessage;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.logging.MessageHandler;
import net.mapeadores.util.logicaloperation.LogicalOperationConstants;
import net.mapeadores.util.logicaloperation.LogicalOperationParser;
import net.mapeadores.util.logicaloperation.Operand;
import net.mapeadores.util.logicaloperation.OperandException;
import net.mapeadores.util.request.RequestMap;
import net.mapeadores.util.text.CleanedString;
import net.mapeadores.util.text.StringUtils;
import net.mapeadores.util.text.TextConstants;
import net.scrutari.datauri.ScrutariDataURI;
import net.scrutari.db.api.AttributeKeyCoder;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDB;
import net.scrutari.db.api.codes.CodeList;
import net.scrutari.db.tools.codes.CodeParser;
import net.scrutari.db.tools.codes.CodesUtils;
import net.scrutari.searchengine.api.operands.filters.UnknownOperandTypeException;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.Category;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.PertinencePonderation;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.tools.OperandCheck;
import net.scrutari.searchengine.tools.operands.filters.FilterOperandConverterEngine;
import net.scrutari.searchengine.tools.operands.filters.OperandConverterParameters;
import net.scrutari.searchengine.tools.operands.search.LogicalSearchOperationEngine;
import net.scrutari.searchengine.tools.operands.search.RawOperation;
import net.scrutari.searchengine.tools.operands.search.RawSearchOperationEngine;
import net.scrutari.searchengine.tools.options.OptionsUtils;
import net.scrutari.searchengine.tools.options.PertinencePonderationBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;
import net.scrutari.searchengine.tools.options.SearchOptionsDefBuilder;
import net.scrutari.supermotcle.api.Supermotcle;
import net.scrutari.supermotcle.api.SupermotcleHolder;


/**
 *
 * @author Vincent Calame
 */
public final class RequestMapUtils implements Parameters {

    private RequestMapUtils() {
    }

    public static StartParameters getStartParameters(RequestMap requestMap, GlobalSearchOptions globalSearchOptions, WarningHandler warningHandler) {
        String startParam = requestMap.getParameter(START);
        if (startParam == null) {
            return StartParameters.NONE;
        }
        int start;
        try {
            start = Integer.parseInt(startParam);
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(START);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, startParam);
            return StartParameters.NONE;
        }
        if (start < 1) {
            return StartParameters.NONE;
        }
        String limitParam = requestMap.getParameter(LIMIT);
        int limit = -1;
        if (limitParam != null) {
            try {
                limit = Integer.parseInt(limitParam);
                if (limit < 1) {
                    limit = -1;
                }
            } catch (NumberFormatException nfe) {
                warningHandler.setCurrentParameter(LIMIT);
                warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, limitParam);
            }
        }
        int categoryCount = globalSearchOptions.getCategoryCount();
        if (categoryCount == 0) {
            return StartParameters.newGlobalInstance(start, limit);
        }
        warningHandler.setCurrentParameter(STARTTYPE);
        String typeParam = requestMap.getParameter(STARTTYPE);
        if ((typeParam == null) || (typeParam.equals(GLOBAL_STARTTYPE))) {
            return StartParameters.newGlobalInstance(start, limit);
        }
        if (typeParam.equals(IN_ALL_STARTTYPE)) {
            return StartParameters.newInAllInstance(start, limit);
        }
        if (typeParam.startsWith(IN_STARTTYPE_PREFIX)) {
            String[] tokens = StringUtils.getTechnicalTokens(typeParam.substring(IN_STARTTYPE_PREFIX.length()), true);
            Set<Integer> rankSet = new HashSet<Integer>();
            int length = tokens.length;
            for (int i = 0; i < length; i++) {
                String token = tokens[i];
                Category category = globalSearchOptions.getCategoryByName(token);
                if (category == null) {
                    warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, token);
                } else {
                    rankSet.add(category.getCategoryDef().getRank());
                }
            }
            return StartParameters.newInSelectionInstance(start, limit, rankSet);
        }
        warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, typeParam);
        return StartParameters.newGlobalInstance(start, limit);
    }

    public static String getLangListParameterValue(RequestMap requestMap) {
        String param_langs = requestMap.getParameter(LANG_LIST);
        if (param_langs == null) {
            param_langs = requestMap.getParameter("langs");
        }
        return param_langs;
    }

    public static Lang[] getLangArray(RequestMap requestMap) {
        String param_langs = requestMap.getParameter(LANG_LIST);
        if (param_langs == null) {
            param_langs = requestMap.getParameter("langs");
        }
        if (param_langs == null) {
            return null;
        }
        Lang[] langArray = LangListUtils.toCleanLangArray(param_langs);
        if (langArray.length == 0) {
            return null;
        }
        return langArray;
    }

    public static Lang getLangUi(RequestMap requestMap, WarningHandler warningHandler) {
        String langUi = requestMap.getParameter(LANG_UI);
        if (langUi == null) {
            return null;
        }
        try {
            Lang lang = Lang.parse(langUi);
            return lang;
        } catch (ParseException lie) {
            warningHandler.setCurrentParameter(LANG_UI);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, langUi);
            return null;
        }
    }

    public static int getLimit(RequestMap requestMap, WarningHandler warningHandler) {
        String limit = requestMap.getParameter(LIMIT);
        if (limit == null) {
            return -1;
        }
        try {
            int limitInteger = Integer.parseInt(limit);
            if (limitInteger > 0) {
                return limitInteger;
            } else {
                return -1;
            }
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(LIMIT);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, limit);
            return -1;
        }
    }

    public static int getStart(RequestMap requestMap, WarningHandler warningHandler) {
        String start = requestMap.getParameter(START);
        if (start == null) {
            return - 1;
        }
        try {
            int globalStartInteger = Integer.parseInt(start);
            if (globalStartInteger > 0) {
                return globalStartInteger;
            } else {
                return -1;
            }
        } catch (NumberFormatException nfe) {
            warningHandler.setCurrentParameter(START);
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, start);
            return -1;
        }
    }

    public static String getQIdString(RequestMap requestMap) {
        String qIdString = requestMap.getParameter(Q_ID);
        if (qIdString == null) {
            qIdString = requestMap.getParameter("idrecherche");
        }
        return qIdString;
    }

    public static DataFilter getDataFilter(RequestMap requestMap) {
        DataFilter dataFilter = new DataFilter();
        String dataString = requestMap.getParameter(DATA);
        if (dataString != null) {
            dataFilter.buildFromString(dataString);
        }
        return dataFilter;
    }

    public static boolean isAllLog(RequestMap requestMap) {
        String logValue = requestMap.getParameter(Parameters.LOG);
        if (logValue == null) {
            return false;
        }
        if (logValue.equals(Parameters.ALL_LOG)) {
            return true;
        } else {
            return false;
        }
    }

    public static SearchOperation getSearchOperation(RequestMap requestMap, AttributeKeyCoder attributeKeyCoder, WarningHandler warningHandler) throws ErrorMessageException {
        String modeString = requestMap.getParameter(Q_MODE);
        if (modeString == null) {
            modeString = INTERSECTION_Q_MODE;
        }
        if (modeString.equals(OPERATION_Q_MODE)) {
            return fromLogicalOperation(requestMap, attributeKeyCoder, warningHandler);
        }
        short mainOperator;
        if (modeString.equals(UNION_Q_MODE)) {
            mainOperator = LogicalOperationConstants.UNION_OPERATOR;
        } else {
            mainOperator = LogicalOperationConstants.INTERSECTION_OPERATOR;
        }
        int defaultSearchType = getDefaultSearchType(requestMap, warningHandler);

        String q = requestMap.getParameter(Q);
        if (q == null) {
            q = requestMap.getParameter("recherche");
            if (q != null) {
                warningHandler.addWarning(WarningKeys.OBSOLETE_PARAMETER_NAME, "recherche");
            }
        }
        List<RawOperation> rawOperationList = checkRawOperationList(requestMap, mainOperator);
        if (q == null) {
            if (rawOperationList == null) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.UNSUPPORTED_MISSING_PARAMETER, Q);
            } else if (rawOperationList.isEmpty()) {
                throw new ErrorMessageException(ParameterErrorMessageKeys.EMPTY_PARAMETER, Q);
            }
        } else {
            q = q.trim();
            if (q.length() == 0) {
                if ((rawOperationList == null) || (rawOperationList.isEmpty())) {
                    throw new ErrorMessageException(ParameterErrorMessageKeys.EMPTY_PARAMETER, Q);
                }
            } else {
                RawOperation rawOperation = new RawOperation(q, "", mainOperator);
                if (rawOperationList == null) {
                    rawOperationList = Collections.singletonList(rawOperation);
                } else {
                    rawOperationList.add(0, rawOperation);
                }
            }
        }
        SearchMessageHandler handler = new SearchMessageHandler(Q, "");
        SearchOperation searchOperation = RawSearchOperationEngine.parse(rawOperationList, handler, attributeKeyCoder, defaultSearchType, mainOperator);
        if (handler.hasError()) {
            throw new ErrorMessageException(handler.toErrorMessage());
        } else if (searchOperation == null) {
            throw new ErrorMessageException(ParameterErrorMessageKeys.EMPTY_PARAMETER, LogUtils.toValue(Q, q));
        } else {
            return searchOperation;
        }
    }

    private static List<RawOperation> checkRawOperationList(RequestMap requestMap, short mainOperator) {
        List<RawOperation> rawOperationList = null;
        for (String paramName : requestMap.getParameterNameSet()) {
            if (paramName.startsWith(Q_PREFIX)) {
                String qName = paramName.substring(Q_PREFIX.length());
                if (qName.length() > 0) {
                    if (rawOperationList == null) {
                        rawOperationList = new ArrayList<RawOperation>();
                    }
                    String q = requestMap.getParameter(paramName).trim();
                    if (q.length() > 0) {
                        String scope = requestMap.getParameter(Q_SCOPE_PREFIX + qName);
                        String mode = requestMap.getParameter(Q_MODE_PREFIX + qName);
                        short operator = mainOperator;
                        if (mode != null) {
                            if (mode.equals(UNION_Q_MODE)) {
                                operator = LogicalOperationConstants.UNION_OPERATOR;
                            } else if (mode.equals(INTERSECTION_Q_MODE)) {
                                operator = LogicalOperationConstants.INTERSECTION_OPERATOR;
                            }
                        }
                        rawOperationList.add(new RawOperation(q, scope, operator));
                    }
                }
            }
        }
        return rawOperationList;
    }

    private static SearchOperation fromLogicalOperation(RequestMap requestMap, AttributeKeyCoder attributeKeyCoder, WarningHandler warningHandler) throws ErrorMessageException {
        String q = requestMap.getParameter(Q);
        if (q == null) {
            throw new ErrorMessageException(ParameterErrorMessageKeys.UNSUPPORTED_MISSING_PARAMETER, Q);
        }
        q = q.trim();
        if (q.length() == 0) {
            throw new ErrorMessageException(ParameterErrorMessageKeys.EMPTY_PARAMETER, Q);
        }
        SearchMessageHandler handler = new SearchMessageHandler(Q, q);
        SearchOperation searchOperation = LogicalSearchOperationEngine.parse(q, handler, attributeKeyCoder);
        if (handler.hasError()) {
            throw new ErrorMessageException(handler.toErrorMessage());
        } else if (searchOperation == null) {
            throw new ErrorMessageException(ParameterErrorMessageKeys.EMPTY_PARAMETER, LogUtils.toValue(Q, q));
        } else {
            return searchOperation;
        }
    }

    public static int getDefaultSearchType(RequestMap requestMap, WarningHandler warningHandler) {
        String wildchar = requestMap.getParameter(WILDCHAR);
        if ((wildchar == null) || (wildchar.length() == 0)) {
            return TextConstants.STARTSWITH;
        } else if (wildchar.equals(END_WILDCHAR)) {
            return TextConstants.STARTSWITH;
        } else if (wildchar.equals(BOTH_WILDCHAR)) {
            return TextConstants.CONTAINS;
        } else if (wildchar.equals(START_WILDCHAR)) {
            return TextConstants.ENDSWITH;
        } else if (wildchar.equals(NONE_WILDCHAR)) {
            return TextConstants.MATCHES;
        } else {
            if (warningHandler != null) {
                warningHandler.setCurrentParameter(WILDCHAR);
                warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, wildchar);
            }
            return TextConstants.STARTSWITH;
        }
    }

    public static SearchOptionsDefBuilder getSearchOptionsBuilder(RequestMap requestMap, WarningHandler warningHandler) {
        SearchOptionsDefBuilder searchOptionsDefBuilder = new SearchOptionsDefBuilder();
        searchOptionsDefBuilder.setLangUi(getLangUi(requestMap, warningHandler));
        String langListParam = getLangListParameterValue(requestMap);
        if (langListParam != null) {
            String[] tokens = StringUtils.getTechnicalTokens(langListParam, true);
            for (String token : tokens) {
                try {
                    Lang lang = Lang.parse(token);
                    searchOptionsDefBuilder.addFilterLang(lang);
                } catch (ParseException mce) {
                }
            }
        }
        searchOptionsDefBuilder.setPertinencePonderation(getPertinencePonderation(requestMap, warningHandler));
        return searchOptionsDefBuilder;
    }

    private static PertinencePonderation getPertinencePonderation(RequestMap requestMap, WarningHandler warningHandler) {
        String ponderationString = requestMap.getParameter(PONDERATION);
        if (ponderationString == null) {
            return null;
        }
        warningHandler.setCurrentParameter(PONDERATION);
        String[] ponderationTokenArray = StringUtils.getTechnicalTokens(ponderationString, true);
        if (ponderationTokenArray.length != 3) {
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, ponderationString);
            return null;
        }
        try {
            int occurrence = Integer.parseInt(ponderationTokenArray[0]);
            int date = Integer.parseInt(ponderationTokenArray[1]);
            int origin = Integer.parseInt(ponderationTokenArray[2]);
            return PertinencePonderationBuilder.toPertinencePonderation(occurrence, date, origin);
        } catch (NumberFormatException nfe) {
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, ponderationString);
            return null;
        }
    }

    public static Supermotcle getSupermotcle(ScrutariSession scrutariSession, RequestMap requestMap) {
        String supermotcleString = requestMap.getParameter("supermotcle");
        if (supermotcleString == null) {
            return null;
        }
        int idx = supermotcleString.indexOf("/");
        if (idx == -1) {
            return null;
        }
        Lang lang;
        int code;
        try {
            lang = Lang.parse(supermotcleString.substring(0, idx));
        } catch (ParseException pe) {
            return null;
        }
        try {
            code = Integer.parseInt(supermotcleString.substring(idx + 1));
        } catch (NumberFormatException nfe) {
            return null;
        }
        SupermotcleHolder supermotcleHolder = scrutariSession.getSupermotcleProvider().getSupermotcleHolder(lang);
        if (supermotcleHolder == null) {
            return null;
        }
        Supermotcle supermotcle = supermotcleHolder.getSupermotcleByCode(code);
        if (supermotcle == null) {
            return null;
        }
        return supermotcle;
    }

    public static String getOrigin(RequestMap requestMap) {
        String origin = requestMap.getParameter(ORIGIN);
        if (origin == null) {
            origin = requestMap.getParameter("site");
        }
        if (origin == null) {
            return "";
        } else {
            return origin;
        }
    }

    public static SearchOptions getSearchOptions(ScrutariSession scrutariSession, RequestMap requestMap, WarningHandler warningHandler, boolean reduceThesaurus) {
        ScrutariDB scrutariDB = scrutariSession.getScrutariDB();
        DataAccess dataAccess = scrutariDB.openDataAccess();
        SearchOptionsDefBuilder searchOptionsDefBuilder = RequestMapUtils.getSearchOptionsBuilder(requestMap, warningHandler);
        searchOptionsDefBuilder.addScrutariDataURIFromCodeList(dataAccess, ScrutariDataURI.BASEURI_TYPE, getBaseCodeList(dataAccess, requestMap, warningHandler));
        searchOptionsDefBuilder.addScrutariDataURIFromCodeList(dataAccess, ScrutariDataURI.THESAURUSURI_TYPE, getThesaurusCodeList(dataAccess, requestMap, warningHandler));
        searchOptionsDefBuilder.addScrutariDataURIFromCodeList(dataAccess, ScrutariDataURI.CORPUSURI_TYPE, getCorpusCodeList(dataAccess, requestMap, warningHandler));
        checkCategoryList(searchOptionsDefBuilder, requestMap, scrutariSession.getGlobalSearchOptions(), warningHandler);
        SearchOptionsBuilder searchOptionsBuilder = new SearchOptionsBuilder(searchOptionsDefBuilder);
        BuildingUtils.computeReduction(searchOptionsBuilder, dataAccess, scrutariSession.getGlobalSearchOptions());
        OperandConverterParameters operandConverterParameters = new OperandConverterParameters();
        operandConverterParameters.setDataAccess(dataAccess);
        operandConverterParameters.setFicheSearchResultHolder(scrutariSession);
        operandConverterParameters.setMessageHandler(warningHandler.getCodeMessageHandler());
        operandConverterParameters.setUriTreeProvider(new AddURITreeProvider(scrutariSession.getSctServer().getSctServerStorage()));
        checkFilters(searchOptionsBuilder, requestMap, warningHandler, operandConverterParameters);
        Supermotcle supermotcle = RequestMapUtils.getSupermotcle(scrutariSession, requestMap);
        if (supermotcle != null) {
            //@todo :
        }
        if (reduceThesaurus) {
            searchOptionsBuilder.reduceThesaurusList(OptionsUtils.getSelectionThesaurusCodeList(scrutariDB.getStats(), dataAccess, searchOptionsBuilder.getCurrentCorpusList()));
        }
        dataAccess.close();
        SearchOptions searchOptions = searchOptionsBuilder.toSearchOptions();
        return searchOptions;
    }

    private static void checkFilters(SearchOptionsBuilder searchOptionsBuilder, RequestMap requestMap, WarningHandler warningHandler, OperandConverterParameters operandConverterParameters) {
        CodeList corpusCodeList = searchOptionsBuilder.getCurrentCorpusList();
        for (String paramName : requestMap.getParameterNameSet()) {
            boolean isObsoleteParamName = false;
            String checkedParamName = paramName;
            if ((paramName.equals("indexation")) || (paramName.equals("flt-indexation"))) {
                checkedParamName = "flt-motcle";
                warningHandler.setCurrentParameter(paramName);
                warningHandler.addWarning(WarningKeys.OBSOLETE_PARAMETER_NAME, paramName);
                isObsoleteParamName = true;
            }
            if (checkedParamName.startsWith(FILTER)) {
                String defaultOperandType;
                if (checkedParamName.equals(FILTER)) {
                    defaultOperandType = null;
                } else if (checkedParamName.startsWith(FILTER_PREFIX)) {
                    defaultOperandType = checkedParamName.substring(FILTER_PREFIX.length());
                } else {
                    continue;
                }
                if (!isObsoleteParamName) {
                    warningHandler.setCurrentParameter(paramName);
                }
                String[] values = requestMap.getParameterValues(paramName);
                for (String value : values) {
                    value = value.trim();
                    if (value.length() == 0) {
                        continue;
                    }
                    Operand operand = parseOperand(value, warningHandler);
                    if (operand != null) {
                        try {
                            OperandCheck operandCheck = FilterOperandConverterEngine.convert(operand, corpusCodeList, defaultOperandType, operandConverterParameters);
                            if (!operandCheck.equals(OperandCheck.UNDEFINED_OPERANDECHECK)) {
                                searchOptionsBuilder.addFicheOperandCheck(operandCheck);
                            }
                        } catch (UnknownOperandTypeException uote) {
                            warningHandler.addWarning(WarningKeys.UNKNOW_OPERAND_TYPE, paramName);
                        }
                    }
                }
            }
        }
    }

    private static void checkCategoryList(SearchOptionsDefBuilder searchOptionsDefBuilder, RequestMap requestMap, GlobalSearchOptions globalSearchOptions, WarningHandler warningHandler) {
        String values = requestMap.getParameter(CATEGORY_LIST);
        if (values == null) {
            return;
        }
        values = values.trim();
        if (values.length() == 0) {
            return;
        }
        warningHandler.setCurrentParameter(CATEGORY_LIST);
        String[] tokens = StringUtils.getTechnicalTokens(values, false);
        int length = tokens.length;
        if (length == 0) {
            warningHandler.addWarning(WarningKeys.ONLYSEPARATORS_PARAMETER_VALUE, values);
        }
        List<CleanedString> list = new ArrayList<CleanedString>();
        for (int i = 0; i < length; i++) {
            CleanedString cs = CleanedString.newInstance(tokens[i]);
            if (cs != null) {
                list.add(cs);
                if (globalSearchOptions.getCategoryByName(cs.toString()) == null) {
                    warningHandler.addWarning(WarningKeys.UNKNOWN_PARAMETER_VALUE, cs.toString());
                }
            }
        }
        if (!list.isEmpty()) {
            searchOptionsDefBuilder.addAttribute(BuildingUtils.CATEGORY_LIST_KEY, list);
        }
    }

    public static CodeList getCorpusCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(CORPUS_LIST);
        warningHandler.setCurrentParameter(CORPUS_LIST);
        if (param == null) {
            param = requestMap.getParameter("codescorpus");
            warningHandler.setCurrentParameter("codescorpus");
        }
        return toCodeList(dataAccess, param, warningHandler, "corpus:", ScrutariDataURI.CORPUSURI_TYPE);
    }

    public static CodeList getThesaurusCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(THESAURUS_LIST);
        warningHandler.setCurrentParameter(THESAURUS_LIST);
        return toCodeList(dataAccess, param, warningHandler, "thesaurus:", ScrutariDataURI.THESAURUSURI_TYPE);
    }

    public static CodeList getBaseCodeList(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        String param = requestMap.getParameter(BASE_LIST);
        warningHandler.setCurrentParameter(BASE_LIST);
        if (param == null) {
            param = requestMap.getParameter("codesbases");
            warningHandler.setCurrentParameter("codesbases");
        }
        return toCodeList(dataAccess, param, warningHandler, "base:", ScrutariDataURI.BASEURI_TYPE);
    }

    /**
     * Retourne null si la chaine est nulle ou blanche, si la chaine ne comprend
     * aucun caractère correct, retourne une liste vide
     */
    public static CodeList toCodeList(DataAccess dataAccess, String values, WarningHandler warningHandler, String uriBase, short uriType) {
        if (values == null) {
            return null;
        }
        values = values.trim();
        if (values.length() == 0) {
            return null;
        }
        String[] tokens = StringUtils.getTechnicalTokens(values, false);
        int length = tokens.length;
        if (length == 0) {
            warningHandler.addWarning(WarningKeys.ONLYSEPARATORS_PARAMETER_VALUE, values);
            return CodesUtils.EMPTY_CODELIST;
        }
        Set<Integer> codeSet = new LinkedHashSet<Integer>();
        for (int i = 0; i < length; i++) {
            int code = CodeParser.toCode(dataAccess, tokens[i], warningHandler.getCodeMessageHandler(), uriType);
            if (code != -1) {
                codeSet.add(code);
            }
        }
        return CodesUtils.toCodeList(codeSet);
    }

    public static IntitulesFilter getIntitulesFilter(RequestMap requestMap) {
        String intitules = requestMap.getParameter(INTITULES);
        if (intitules == null) {
            return null;
        }
        IntitulesFilter intitulesFilter = new IntitulesFilter();
        intitulesFilter.buildFromString(intitules);
        return intitulesFilter;
    }

    public static FicheFieldsFilter getFicheFieldsFilter(RequestMap requestMap) {
        FicheFieldsFilter ficheFieldsFilter = new FicheFieldsFilter();
        String ficheFields = requestMap.getParameter(FICHE_FIELDS);
        if (ficheFields == null) {
            ficheFields = requestMap.getParameter("prop");
        }
        if (ficheFields != null) {
            ficheFieldsFilter.buildFromString(ficheFields);
        }
        for (String paramName : requestMap.getParameterNameSet()) {
            if (paramName.startsWith(FIELD_PREFIX)) {
                String field = paramName.substring(FIELD_PREFIX.length());
                String value = requestMap.getParameter(paramName);
                if (field.equals("titre")) {
                    ficheFieldsFilter.setTitrePropertyName(value);
                } else if (field.equals("soustitre")) {
                    ficheFieldsFilter.setSoustitrePropertyName(value);
                } else if (field.equals("href")) {
                    ficheFieldsFilter.setHrefPropertyName(value);
                } else if (field.equals("annee")) {
                    ficheFieldsFilter.setAnneePropertyName(value);
                } else {
                    try {
                        AttributeKey attributeKey = AttributeKey.parse(field);
                        ficheFieldsFilter.putAttributePropertyName(attributeKey, value);
                    } catch (ParseException pe) {

                    }
                }
            }
        }
        return ficheFieldsFilter;
    }


    public static MotcleFieldsFilter getMotcleFieldsFilter(RequestMap requestMap) {
        MotcleFieldsFilter motcleFieldsFilter = new MotcleFieldsFilter();
        String ficheFields = requestMap.getParameter(MOTCLE_FIELDS);
        if (ficheFields != null) {
            motcleFieldsFilter.buildFromString(ficheFields);
        }
        return motcleFieldsFilter;
    }

    public static CountFilter getCountFilter(DataAccess dataAccess, RequestMap requestMap, WarningHandler warningHandler) {
        CountFilter countFilter = new CountFilter();
        countFilter.setLangArray(getLangArray(requestMap));
        countFilter.setCorpusCodeList(getCorpusCodeList(dataAccess, requestMap, warningHandler));
        countFilter.setBaseCodeList(getBaseCodeList(dataAccess, requestMap, warningHandler));
        return countFilter;
    }

    private static Operand parseOperand(String operationString, WarningHandler warningHandler) {
        try {
            return LogicalOperationParser.parse(operationString);
        } catch (OperandException pe) {
            warningHandler.addWarning(WarningKeys.MALFORMED_PARAMETER_VALUE, operationString + "(" + pe.getErrorKey() + " / " + pe.getCharIndex() + ")");
            return null;
        }
    }


    private static class SearchMessageHandler implements MessageHandler {

        private final String sourceString;
        private final String parameter;
        private final MessageListBuilder errorListBuilder = new MessageListBuilder();

        private SearchMessageHandler(String parameter, String sourceString) {
            this.parameter = parameter;
            this.sourceString = sourceString;
        }

        @Override
        public void addMessage(String category, Message message) {
            errorListBuilder.addMessage(message);
        }

        private boolean hasError() {
            return (errorListBuilder.size() > 0);
        }

        private CommandMessage toErrorMessage() {
            return CommandMessage.error(ParameterErrorMessageKeys.WRONG_OPERATION_SYNTAX, LogUtils.toValue(parameter, sourceString), errorListBuilder.toMessageList());

        }

    }

}
