/* SctServer - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.collect;

import fr.exemole.sctserver.api.collect.CollectResult;
import fr.exemole.sctserver.api.collect.ScrutariInfoContent;
import fr.exemole.sctserver.api.collect.ScrutariSource;
import fr.exemole.sctserver.api.collect.ScrutariSourceDef;
import fr.exemole.sctserver.api.collect.StateErrorException;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.api.storage.SctStorageException;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.HttpURLConnection;
import java.net.MalformedURLException;
import java.net.URL;
import java.net.URLConnection;
import java.util.ArrayList;
import java.util.List;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import net.mapeadores.util.exceptions.NestedLibraryException;
import net.mapeadores.util.exceptions.SwitchException;
import net.mapeadores.util.logging.MultiMessageHandler;
import net.mapeadores.util.primitives.DateInteger;
import net.mapeadores.util.primitives.DateIntegerException;
import net.mapeadores.util.xml.XMLUtils;
import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;
import org.xml.sax.SAXException;


/**
 *
 * @author Vincent Calame
 */
public final class CollectEngine {

    private final ScrutariSource scrutariSource;
    private final SctServerStorage sctServerStorage;
    private final MultiMessageHandler messageHandler;
    private final ScrutariSourceDef scrutariSourceDef;
    private final boolean forceDataDownload;
    private final List<File> tempFileList = new ArrayList<File>();
    private short state = ScrutariSource.OK_STATE;
    private int lastUpdateDate = 0;


    static {
        CollectUtils.initSslFactory();
    }

    private CollectEngine(ScrutariSource scrutariSource, SctServerStorage sctServerStorage, MultiMessageHandler messageHandler, boolean forceDataDownload) {
        this.sctServerStorage = sctServerStorage;
        this.scrutariSource = scrutariSource;
        this.messageHandler = messageHandler;
        this.forceDataDownload = forceDataDownload;
        this.scrutariSourceDef = scrutariSource.getScrutariSourceDef();
    }


    private void run() {
        messageHandler.setCurrentURI(scrutariSourceDef.getName());
        int urlCount = scrutariSourceDef.getUrlCount();
        boolean error = false;
        for (int i = 0; i < urlCount; i++) {
            short type = scrutariSourceDef.getUrlType(i);
            try {
                switch (type) {
                    case ScrutariSourceDef.INFO_TYPE:
                        fromInfoType(i);
                        break;
                    case ScrutariSourceDef.SCRUTARIDATA_TYPE:
                        fromDataType(i);
                        break;
                    default:
                        throw new SwitchException("Unknown type: " + type);
                }
            } catch (StateErrorException stateErrorException) {
                state = stateErrorException.getErrorState();
                lastUpdateDate = 0;
                error = true;
            }
            if (error) {
                break;
            }
        }
        File[] tempFileArray = tempFileList.toArray(new File[tempFileList.size()]);
        if (!error) {
            sctServerStorage.saveScrutariData(scrutariSource.getScrutariSourceDef().getName(), tempFileArray);
        }
        for (File file : tempFileArray) {
            file.delete();
        }
    }

    public static CollectResult collectData(ScrutariSource scrutariSource, SctServerStorage sctServerStorage, MultiMessageHandler messageHandler, boolean forceDataDownload) {
        messageHandler.setCurrentURI(scrutariSource.getScrutariSourceDef().getName());
        CollectEngine engine = new CollectEngine(scrutariSource, sctServerStorage, messageHandler, forceDataDownload);
        engine.run();
        return CollectUtils.toCollectResult(engine.state, engine.lastUpdateDate);
    }


    private void fromDataType(int index) throws StateErrorException {
        int frequency = scrutariSourceDef.getFrequency(index);
        int currentDate = DateInteger.current();
        if ((!forceDataDownload) && (scrutariSource.getLastUpdate() > DateInteger.roll(currentDate, -frequency))) {
            lastUpdateDate = Math.max(lastUpdateDate, scrutariSource.getLastUpdate());
            return;
        }
        String name = scrutariSourceDef.getName();
        File tempFile = createTempFile(name);
        try (OutputStream os = new FileOutputStream(tempFile)) {
            downloadData(os, scrutariSourceDef.getUrl(index), messageHandler);
        } catch (IOException ioe) {
            tempFile.delete();
            throw new SctStorageException(ioe);
        }
        tempFileList.add(tempFile);
        lastUpdateDate = currentDate;
    }

    private void fromInfoType(int index) throws StateErrorException {
        ScrutariInfoContent scrutariInfoContent = getInfoContent(scrutariSourceDef.getUrl(index), messageHandler);
        int lastUpdate = scrutariInfoContent.getLastUpdateDateInteger();
        if (!forceDataDownload) {
            if (lastUpdate < scrutariSource.getLastUpdate()) {
                lastUpdateDate = Math.max(lastUpdateDate, scrutariSource.getLastUpdate());
            }
        }
        String name = scrutariSourceDef.getName();
        File tempFile = createTempFile(name);
        try (OutputStream os = new FileOutputStream(tempFile)) {
            int dataCount = scrutariInfoContent.getDataUrlCount();
            for (int i = 0; i < dataCount; i++) {
                URL dataUrl = scrutariInfoContent.getDataUrl(i);
                downloadData(os, dataUrl, messageHandler);
            }
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
        tempFileList.add(tempFile);
        lastUpdateDate = DateInteger.current();
    }

    private static File createTempFile(String name) {
        try {
            File tempFile = File.createTempFile(name, ".tmp");
            tempFile.deleteOnExit();
            return tempFile;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    private static void downloadData(OutputStream outputStream, URL dataUrl, MultiMessageHandler messageHandler) throws IOException, StateErrorException {
        URLConnection con = null;
        try {
            con = dataUrl.openConnection();
            if (!CollectUtils.testConnection(con, messageHandler)) {
                throw new StateErrorException(ScrutariSource.DATA_BROKENURL_STATE);
            }
        } catch (IOException ioe) {
            CollectUtils.warningDownload(messageHandler, dataUrl, ioe.getMessage());
            throw new StateErrorException(ScrutariSource.DATA_NOCONNECTION_STATE);
        }
        try (InputStream is = con.getInputStream()) {
            byte[] buffer = new byte[1024];
            int size;
            while ((size = is.read(buffer, 0, 1024)) > 0) {
                outputStream.write(buffer, 0, size);
            }
        } catch (IOException ioe) {
            CollectUtils.warningDownload(messageHandler, dataUrl, ioe.getMessage());
            throw new StateErrorException(ScrutariSource.DATA_DOWNLOADERROR_STATE);
        }

    }

    private static ScrutariInfoContent getInfoContent(URL url, MultiMessageHandler messageHandler) throws StateErrorException {
        URLConnection con = null;
        try {
            con = (HttpURLConnection) url.openConnection();
            if (!CollectUtils.testConnection(con, messageHandler)) {
                throw new StateErrorException(ScrutariSource.INFO_BROKENURL_STATE);
            }
        } catch (IOException ioe) {
            CollectUtils.warningDownload(messageHandler, url, ioe.getMessage());
            throw new StateErrorException(ScrutariSource.INFO_NOCONNECTION_STATE);
        }
        DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();
        DocumentBuilder db = null;
        try {
            db = dbf.newDocumentBuilder();
        } catch (ParserConfigurationException e) {
            throw new NestedLibraryException(e);
        }
        Document doc = null;
        try {
            doc = db.parse(con.getInputStream());
        } catch (IOException ioe) {
            CollectUtils.warningDownload(messageHandler, url, ioe.getMessage());
            throw new StateErrorException(ScrutariSource.INFO_DOWNLOADERROR_STATE);
        } catch (SAXException saxe) {
            CollectUtils.warningDownload(messageHandler, url, saxe.getMessage());
            throw new StateErrorException(ScrutariSource.INFO_NOTXML_STATE);
        }
        String infoUrlString = url.toString();
        int idx = infoUrlString.lastIndexOf('/');
        String urlRoot = infoUrlString.substring(0, idx + 1);
        return readScrutariInfoDOM(doc.getDocumentElement(), urlRoot);
    }

    private static ScrutariInfoContent readScrutariInfoDOM(Element element_xml, String urlRoot) throws StateErrorException {
        String lastDateString = null;
        List<String> dataUrlStringList = new ArrayList<String>();
        NodeList nodeList = element_xml.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            Node node = nodeList.item(i);
            if (node.getNodeType() == Node.ELEMENT_NODE) {
                Element el = (Element) node;
                String tagname = el.getTagName();
                if (tagname.equals("date")) {
                    lastDateString = XMLUtils.getData(el);
                } else if ((tagname.equals("path")) || (tagname.equals("scrutaridata-url"))) {
                    String pathString = XMLUtils.getData(el);
                    if (pathString.length() > 0) {
                        dataUrlStringList.add(pathString);
                    }
                }
            }
        }
        if ((dataUrlStringList.isEmpty()) || (lastDateString == null)) {
            throw new StateErrorException(ScrutariSource.INFO_WRONGXML_STATE);
        }
        int lastUpdate;
        try {
            lastUpdate = DateInteger.parse(lastDateString);
        } catch (DateIntegerException ide) {
            throw new StateErrorException(ScrutariSource.INFO_WRONGXML_STATE);
        }
        List<URL> urlList = new ArrayList<URL>();
        for (String urlString : dataUrlStringList) {
            try {
                URL dataUrl = buildDataUrl(urlString, urlRoot);
                urlList.add(dataUrl);
            } catch (MalformedURLException mue) {
                throw new StateErrorException(ScrutariSource.INFO_WRONGXML_STATE);
            }
        }
        return new InternalScrutariInfoContent(lastUpdate, urlList.toArray(new URL[urlList.size()]));
    }

    private static URL buildDataUrl(String pathString, String urlRoot) throws MalformedURLException {
        try {
            URL url = new URL(pathString);
            return url;
        } catch (MalformedURLException mue) {
            URL url = new URL(urlRoot + pathString);
            return url;
        }
    }


    private static class InternalScrutariInfoContent implements ScrutariInfoContent {

        private final int lastUpdate;
        private final URL[] urlArray;

        private InternalScrutariInfoContent(int lastUpdate, URL[] urlArray) {
            this.lastUpdate = lastUpdate;
            this.urlArray = urlArray;
        }

        @Override
        public int getLastUpdateDateInteger() {
            return lastUpdate;
        }

        @Override
        public int getDataUrlCount() {
            return urlArray.length;
        }

        @Override
        public URL getDataUrl(int i) {
            return urlArray[i];
        }

    }

}
