/* SctServer - Copyright (c) 2013-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.tools.feed;

import fr.exemole.sctserver.api.ScrutariSession;
import fr.exemole.sctserver.api.feed.SctFeedInfo;
import fr.exemole.sctserver.api.feed.SctFeedOptions;
import fr.exemole.sctserver.request.ParameterErrorMessageKeys;
import fr.exemole.sctserver.request.Parameters;
import static fr.exemole.sctserver.request.Parameters.Q_ID;
import fr.exemole.sctserver.request.RequestMapUtils;
import fr.exemole.sctserver.request.WarningHandler;
import java.text.DateFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Map;
import net.mapeadores.util.localisation.Lang;
import net.mapeadores.util.logging.ErrorMessageException;
import net.mapeadores.util.logging.LogUtils;
import net.mapeadores.util.request.RequestMap;
import net.scrutari.data.BaseData;
import net.scrutari.data.CorpusData;
import net.scrutari.data.FicheData;
import net.scrutari.datauri.BaseURI;
import net.scrutari.datauri.CorpusURI;
import net.scrutari.datauri.FicheURI;
import net.scrutari.datauri.tree.BaseNode;
import net.scrutari.datauri.tree.CorpusNode;
import net.scrutari.datauri.tree.FicheNode;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.db.api.DataAccess;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.searchengine.FicheSearchEngine;
import net.scrutari.searchengine.api.QId;
import net.scrutari.searchengine.api.operands.search.SearchOperation;
import net.scrutari.searchengine.api.options.GlobalSearchOptions;
import net.scrutari.searchengine.api.options.SearchOptions;
import net.scrutari.searchengine.api.options.SearchOptionsDef;
import net.scrutari.searchengine.api.result.FicheSearchResult;
import net.scrutari.searchengine.api.result.FicheSearchResultGroup;
import net.scrutari.searchengine.api.result.FicheSearchResultInfo;
import net.scrutari.searchengine.tools.EligibilityBuffer;
import net.scrutari.searchengine.tools.options.SearchOptionsBuilder;


/**
 *
 * @author Vincent Calame
 */
public final class FicheFeedListEngine {

    private final static long TWO_DAYS = 2 * 24 * 60 * 60 * 1000;
    private final static DateFormat ISO_FORMAT = new SimpleDateFormat("yyyy-MM-dd");

    private FicheFeedListEngine() {
    }

    public static FicheFeedList build(ScrutariSession scrutariSession, SctFeedInfo sctFeedInfo, DataAccess dataAccess, SctFeedOptions feedOptions) throws ErrorMessageException {
        Lang langUi = feedOptions.getLangUi();
        ScrutariDBName scrutariDBName = sctFeedInfo.getLastScrutariDBName(0);
        RequestMap requestMap = feedOptions.getRequestMap();
        FicheSearchResult ficheSearchResult = null;
        SearchOperation searchOperation = null;
        SearchOptions searchOptions = null;
        if (requestMap != null) {
            searchOptions = RequestMapUtils.getSearchOptions(scrutariSession, requestMap, new WarningHandler(), true);
            String qIdString = requestMap.getParameter(Parameters.Q_ID);
            if (qIdString != null) {
                try {
                    QId qId = QId.parse(qIdString);
                    ficheSearchResult = scrutariSession.getSearch(qId);
                    if (ficheSearchResult == null) {
                        throw new ErrorMessageException(ParameterErrorMessageKeys.UNKNOWN_PARAMETER_VALUE, LogUtils.toValue(Q_ID, qIdString));
                    }
                    if (isAllList(requestMap)) {
                        return getRecentQList(scrutariSession, dataAccess, ficheSearchResult, feedOptions, scrutariDBName);
                    }
                } catch (ParseException pe) {
                    throw new ErrorMessageException(ParameterErrorMessageKeys.WRONG_PARAMETER_VALUE, LogUtils.toValue(Q_ID, qIdString));
                }
            } else if (requestMap.getParameter(Parameters.Q) != null) {
                searchOperation = RequestMapUtils.getSearchOperation(requestMap, scrutariSession.getScrutariDB().getAttributeKeyCoder(), null);
            }
        }
        FicheFeedList ficheFeedList = init(scrutariSession, sctFeedInfo, dataAccess, langUi);
        if (searchOperation != null) {
            SearchOptionsBuilder searchOptionsBuilder = SearchOptionsBuilder.derive(searchOptions);
            searchOptionsBuilder.getSearchOptionsDefBuilder().setLangUi(langUi);
            searchOptionsBuilder.reduceFicheList(ficheFeedList);
            ficheSearchResult = FicheSearchEngine.search(scrutariSession.getScrutariDB(), searchOperation, searchOptionsBuilder.toSearchOptions(), scrutariSession.getGlobalSearchOptions(), null, false);
        } else if (searchOptions != null) {
            SearchOptionsDef searchOptionsDef = searchOptions.getSearchOptionsDef();
            EligibilityBuffer eligibilityBuffer = new EligibilityBuffer(dataAccess, searchOptionsDef);
            ficheFeedList.update(eligibilityBuffer, searchOptionsDef.getFilterLangList(), searchOptions.getCorpusCodeList());
        }
        if (ficheSearchResult != null) {
            ficheFeedList.update(ficheSearchResult);
        }
        return ficheFeedList;
    }

    private static FicheFeedList init(ScrutariSession scrutariSession, SctFeedInfo sctFeedInfo, DataAccess dataAccess, Lang langUi) {
        FicheFeedList ficheFeedList = new FicheFeedList(langUi);
        int scrutariDBNameCount = sctFeedInfo.getLastScrutariDBNameCount();
        WrapperHolder wrapperHolder = new WrapperHolder(dataAccess, langUi, scrutariSession.getGlobalSearchOptions());
        for (int h = 0; h < scrutariDBNameCount; h++) {
            ScrutariDBName scrutariDBName = sctFeedInfo.getLastScrutariDBName(h);
            URITree uriTree = scrutariSession.getSctServer().getSctServerStorage().getURITree(scrutariDBName, scrutariSession.getScrutariDataURIChecker());
            if (uriTree != null) {
                int baseCount = uriTree.getBaseCount();
                for (int i = 0; i < baseCount; i++) {
                    BaseNode baseNode = uriTree.getBaseNode(i);
                    BaseWrapper baseWrapper = wrapperHolder.getBaseWrapper(baseNode.getBaseURI());
                    if (baseWrapper == null) {
                        continue;
                    }
                    int corpusCount = baseNode.getCorpusCount();
                    for (int j = 0; j < corpusCount; j++) {
                        CorpusNode corpusNode = baseNode.getCorpusNode(j);
                        CorpusWrapper corpusWrapper = wrapperHolder.getCorpusWrapper(corpusNode.getCorpusURI());
                        if (corpusWrapper == null) {
                            continue;
                        }
                        int ficheCount = corpusNode.getFicheCount();
                        for (int k = 0; k < ficheCount; k++) {
                            FicheNode ficheNode = corpusNode.getFicheNode(k);
                            FicheURI ficheURI = ficheNode.getFicheURI();
                            FicheData ficheData = (FicheData) dataAccess.getScrutariData(ficheURI);
                            if (ficheData == null) {
                                continue;
                            }
                            ficheFeedList.addFicheData(ficheData, corpusWrapper, baseWrapper, scrutariDBName);
                        }
                    }
                }
            }
        }
        return ficheFeedList;
    }

    private static boolean isAllList(RequestMap requestMap) {
        String allString = requestMap.getParameter(Parameters.ALL);
        if (allString == null) {
            return false;
        }
        if ((allString.equals("1")) || (allString.equals("always"))) {
            return true;
        }
        synchronized (ISO_FORMAT) {
            try {
                Date date = ISO_FORMAT.parse(allString);
                return isRecent(date);
            } catch (ParseException pe) {
                return false;
            }
        }
    }

    private static FicheFeedList getRecentQList(ScrutariSession scrutariSession, DataAccess dataAccess, FicheSearchResult ficheSearchResult, SctFeedOptions feedOptions, ScrutariDBName lastScrutariDBName) {
        Lang langUi = feedOptions.getLangUi();
        WrapperHolder wrapperHolder = new WrapperHolder(dataAccess, langUi, scrutariSession.getGlobalSearchOptions());
        FicheFeedList ficheFeedList = new FicheFeedList(langUi);
        int groupCount = ficheSearchResult.getFicheSearchResultGroupCount();
        for (int i = 0; i < groupCount; i++) {
            FicheSearchResultGroup group = ficheSearchResult.getFicheSearchResultGroup(i);
            int ficheCount = group.getFicheSearchResultInfoCount();
            for (int j = 0; j < ficheCount; j++) {
                FicheSearchResultInfo info = group.getFicheSearchResultInfo(j);
                int codefiche = info.getCode();
                FicheData ficheData = dataAccess.getFicheDataByCode(codefiche);
                ficheFeedList.addFicheData(ficheData, wrapperHolder.getCorpusWrapper(ficheData.getCodeCorpus()), wrapperHolder.getBaseWrapper(ficheData.getCodeBase()), lastScrutariDBName);
            }
        }
        return ficheFeedList;
    }

    private static boolean isRecent(Date date) {
        long time = date.getTime();
        long current = System.currentTimeMillis();
        return (current - time < TWO_DAYS);
    }


    private static class WrapperHolder {

        private final Map<Integer, BaseWrapper> baseWrapperMap = new HashMap<Integer, BaseWrapper>();
        private final Map<Integer, CorpusWrapper> corpusWrapperMap = new HashMap<Integer, CorpusWrapper>();
        private final DataAccess dataAccess;
        private final Lang langUi;
        private final GlobalSearchOptions globalSearchOptions;

        private WrapperHolder(DataAccess dataAccess, Lang langUi, GlobalSearchOptions globalSearchOptions) {
            this.dataAccess = dataAccess;
            this.langUi = langUi;
            this.globalSearchOptions = globalSearchOptions;
        }

        private BaseWrapper getBaseWrapper(BaseURI baseURI) {
            BaseData baseData = (BaseData) dataAccess.getScrutariData(baseURI);
            if (baseData == null) {
                return null;
            }
            int codebase = baseData.getCodeBase();
            BaseWrapper baseWrapper = baseWrapperMap.get(codebase);
            if (baseWrapper == null) {
                baseWrapper = BaseWrapper.wrapBase(baseData, langUi);
                baseWrapperMap.put(codebase, baseWrapper);
            }
            return baseWrapper;
        }

        private BaseWrapper getBaseWrapper(int codebase) {
            BaseWrapper baseWrapper = baseWrapperMap.get(codebase);
            if (baseWrapper == null) {
                BaseData baseData = dataAccess.getBaseDataByCode(codebase);
                baseWrapper = BaseWrapper.wrapBase(baseData, langUi);
                baseWrapperMap.put(codebase, baseWrapper);
            }
            return baseWrapper;
        }

        private CorpusWrapper getCorpusWrapper(CorpusURI corpusURI) {
            CorpusData corpusData = (CorpusData) dataAccess.getScrutariData(corpusURI);
            if (corpusData == null) {
                return null;
            }
            int codecorpus = corpusData.getCodeCorpus();
            CorpusWrapper corpusWrapper = corpusWrapperMap.get(codecorpus);
            if (corpusWrapper == null) {
                corpusWrapper = CorpusWrapper.wrapCorpus(corpusData, langUi, globalSearchOptions);
                corpusWrapperMap.put(codecorpus, corpusWrapper);
            }
            return corpusWrapper;
        }

        private CorpusWrapper getCorpusWrapper(int codecorpus) {
            CorpusWrapper corpusWrapper = corpusWrapperMap.get(codecorpus);
            if (corpusWrapper == null) {
                CorpusData corpusData = dataAccess.getCorpusDataByCode(codecorpus);
                corpusWrapper = CorpusWrapper.wrapCorpus(corpusData, langUi, globalSearchOptions);
                corpusWrapperMap.put(codecorpus, corpusWrapper);
            }
            return corpusWrapper;
        }

    }

}
