/* SctServer_Directory - Copyright (c) 2008-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package fr.exemole.sctserver.directory;

import fr.exemole.sctserver.api.conf.ConfDirectory;
import fr.exemole.sctserver.api.feed.SctFeedCache;
import fr.exemole.sctserver.api.storage.FicheSearchCache;
import fr.exemole.sctserver.api.storage.LogStorage;
import fr.exemole.sctserver.api.storage.QIdStorage;
import fr.exemole.sctserver.api.storage.SctServerStorage;
import fr.exemole.sctserver.api.storage.SctStorageException;
import fr.exemole.sctserver.directory.cache.CacheUtils;
import fr.exemole.sctserver.directory.cache.DirectoryDataCacheReaderFactory;
import fr.exemole.sctserver.directory.cache.DirectoryDataCacheWriter;
import fr.exemole.sctserver.directory.cache.DirectoryFicheSearchCache;
import fr.exemole.sctserver.directory.cache.DirectoryLexieCacheReaderFactory;
import fr.exemole.sctserver.directory.cache.DirectoryLexieCacheWriter;
import fr.exemole.sctserver.directory.cache.DirectorySctFeedCache;
import fr.exemole.sctserver.directory.persistent.DirectoryLogStorage;
import fr.exemole.sctserver.directory.persistent.DirectoryQIdStorage;
import fr.exemole.sctserver.tools.conf.factory.ConfDirectoryFactory;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.InputStreamReader;
import java.io.OutputStreamWriter;
import java.io.RandomAccessFile;
import java.text.ParseException;
import java.util.Set;
import java.util.SortedMap;
import java.util.TreeMap;
import java.util.TreeSet;
import java.util.regex.Matcher;
import java.util.regex.Pattern;
import net.mapeadores.util.io.FileUtils;
import net.mapeadores.util.io.IOUtils;
import net.mapeadores.util.primitives.io.DataInputPrimitivesReader;
import net.mapeadores.util.primitives.io.DataOutputPrimitivesWriter;
import net.scrutari.datauri.ScrutariDataURIChecker;
import net.scrutari.datauri.tree.URITree;
import net.scrutari.datauri.tree.io.TsvReader;
import net.scrutari.datauri.tree.io.TsvWriter;
import net.scrutari.db.api.ScrutariDBName;
import net.scrutari.db.api.caches.DataCacheReaderFactory;
import net.scrutari.db.api.caches.DataCacheWriter;
import net.scrutari.db.api.caches.LexieCacheReaderFactory;
import net.scrutari.db.api.caches.LexieCacheWriter;
import net.scrutari.db.tools.AliasManager;


/**
 *
 * @author Vincent Calame
 */
public class DirectorySctServerStorage implements SctServerStorage {

    private final static DataList EMPTY_DATALIST = new InternalDataList(new File[0]);
    private final File varDir;
    private final File cacheDirectory;
    private final File dataDirectory;
    private final File uriDirectory;
    private final ConfDirectory confDirectory;
    private final DirectoryQIdStorage qIdStorage;
    private final DirectoryLogStorage logStorage;

    public DirectorySctServerStorage(File confDir, File varDir) {
        this.varDir = varDir;
        this.confDirectory = ConfDirectoryFactory.fromDirectory(confDir);
        this.cacheDirectory = getDirectory("cache");
        this.dataDirectory = getDirectory("data");
        this.uriDirectory = getDirectory("uri");
        this.qIdStorage = new DirectoryQIdStorage(getDirectory("qid"), cacheDirectory);
        this.logStorage = new DirectoryLogStorage(getDirectory("logs"));
        convert();
    }

    @Override
    public DataCacheWriter getDataCacheWriter(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/scrutaridb");
        dbCacheDirectory.mkdirs();
        return new DirectoryDataCacheWriter(dbCacheDirectory);
    }

    @Override
    public LexieCacheWriter getLexieCacheWriter(ScrutariDBName scrutariDbName) {
        File dbCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/scrutaridb");
        dbCacheDirectory.mkdirs();
        return new DirectoryLexieCacheWriter(dbCacheDirectory);
    }

    @Override
    public SctFeedCache getSctFeedCache(ScrutariDBName scrutariDbName) {
        File feedCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/feed");
        feedCacheDirectory.mkdirs();
        return new DirectorySctFeedCache(feedCacheDirectory);
    }

    @Override
    public FicheSearchCache getFicheSearchCache(ScrutariDBName scrutariDbName) {
        File searchCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/search");
        searchCacheDirectory.mkdirs();
        return new DirectoryFicheSearchCache(searchCacheDirectory);
    }

    @Override
    public void cacheObject(ScrutariDBName scrutariDbName, Object obj) {
        File objectsDir = new File(cacheDirectory, scrutariDbName.getName() + "/objects");
        objectsDir.mkdirs();
        if (obj instanceof AliasManager) {
            File f = new File(objectsDir, "aliasmanager.primitives");
            try (RandomAccessFile dataRandomAccessFile = new RandomAccessFile(f, "rw")) {
                DataOutputPrimitivesWriter dataPrimitivesWriter = new DataOutputPrimitivesWriter(dataRandomAccessFile);
                AliasManager.toPrimitives((AliasManager) obj, dataPrimitivesWriter);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }
    }

    @Override
    public Object getCachedObject(ScrutariDBName scrutariDbName, Class objectClass) {
        File objectsDir = new File(cacheDirectory, scrutariDbName.getName() + "/objects");
        if (!objectsDir.exists()) {
            return null;
        }
        if (objectClass.equals(AliasManager.class)) {
            File f = new File(objectsDir, "aliasmanager.primitives");
            if (!f.exists()) {
                return null;
            }
            try (RandomAccessFile dataRandomAccessFile = new RandomAccessFile(f, "r")) {
                DataInputPrimitivesReader primReader = new DataInputPrimitivesReader(dataRandomAccessFile);
                return AliasManager.fromPrimitives(primReader);
            } catch (IOException ioe) {
                return null;
            }
        }
        return null;
    }

    @Override
    public QIdStorage getQIdStorage() {
        return qIdStorage;
    }

    @Override
    public LogStorage getLogStorage() {
        return logStorage;
    }

    @Override
    public void cleanCache(Set<ScrutariDBName> remainingSet) {
        File[] dirs = cacheDirectory.listFiles();
        int length = dirs.length;
        for (int i = 0; i < length; i++) {
            File dir = dirs[i];
            if (!dir.isDirectory()) {
                continue;
            }
            String name = dir.getName();
            boolean remain;
            try {
                ScrutariDBName dbName = ScrutariDBName.parse(name);
                remain = remainingSet.contains(dbName);
            } catch (ParseException pe) {
                continue;
            }
            if (!remain) {
                try {
                    IOUtils.cleanDirectory(dir);
                    dir.delete();
                } catch (IOException ioe) {
                    throw new SctStorageException(ioe);
                }
            }
        }
    }

    @Override
    public void cleanTransientCache(ScrutariDBName scrutariDbName) {
        File feedCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/feed");
        if (feedCacheDirectory.exists()) {
            try {
                IOUtils.cleanDirectory(feedCacheDirectory);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }
        File searchCacheDirectory = new File(cacheDirectory, scrutariDbName.getName() + "/search");
        if (searchCacheDirectory.exists()) {
            try {
                IOUtils.cleanDirectory(searchCacheDirectory);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }
    }

    @Override
    public ConfDirectory getConfDirectory() {
        return confDirectory;
    }

    private File getDirectory(String name) {
        File directory = new File(varDir, name);
        testDirectory(directory);
        return directory;
    }

    private void testDirectory(File directory) {
        if (!directory.exists()) {
            directory.mkdirs();
        } else if (!directory.isDirectory()) {
            throw new SctStorageException("Not a directory: " + directory.getPath());
        }
    }

    @Override
    public DataList getDataList(String scrutariSourceName) {
        File dir = new File(dataDirectory, scrutariSourceName);
        if (!dir.exists()) {
            return EMPTY_DATALIST;
        }
        SortedMap<Integer, File> map = new TreeMap<Integer, File>();
        Pattern pattern = Pattern.compile("^data-([0-9]+).xml$");
        for (File f : dir.listFiles()) {
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            Matcher matcher = pattern.matcher(name);
            if (matcher.matches()) {
                map.put(Integer.parseInt(matcher.group(1)), f);
            }
        }
        File[] array = map.values().toArray(new File[map.size()]);
        return new InternalDataList(array);
    }

    @Override
    public void saveScrutariData(String scrutariSourceName, File[] tempFileArray) {
        File dir = new File(dataDirectory, scrutariSourceName);
        dir.mkdirs();
        try {
            FileUtils.cleanDirectory(dir);
            int length = tempFileArray.length;
            for (int i = 0; i < length; i++) {
                int increment = i + 1;
                File destination = new File(dir, "data-" + increment + ".xml");
                FileUtils.copyFile(tempFileArray[i], destination);
            }
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public String getContent(String contentName) {
        File contentFile = getContentFile(contentName);
        if (contentFile == null) {
            return "";
        }
        if (!contentFile.exists()) {
            return "";
        }
        try (FileInputStream is = new FileInputStream(contentFile)) {
            String content = IOUtils.toString(is, "UTF-8");
            return content;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void saveContent(String contentName, String content) {
        File contentFile = getContentFile(contentName);
        if (contentFile == null) {
            throw new IllegalArgumentException("Unknown content : " + contentName);
        }
        try (OutputStreamWriter writer = new OutputStreamWriter(new FileOutputStream(contentFile), "UTF-8")) {
            IOUtils.write(content, writer);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public void saveURITree(URITree uriTree, ScrutariDBName scrutariDBName) {
        File file = getUriTreeFile(scrutariDBName);
        try (BufferedWriter writer = new BufferedWriter(new OutputStreamWriter(new FileOutputStream(file), "UTF-8"))) {
            TsvWriter.write(writer, uriTree);
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public URITree getURITree(ScrutariDBName scrutariDBName, ScrutariDataURIChecker checker) {
        File file = getUriTreeFile(scrutariDBName);
        if (!file.exists()) {
            return null;
        }
        try (InputStreamReader reader = new InputStreamReader(new FileInputStream(file), "UTF-8")) {
            URITree uriTree = TsvReader.readURITree(reader, checker);
            return uriTree;
        } catch (IOException ioe) {
            throw new SctStorageException(ioe);
        }
    }

    @Override
    public ScrutariDBName[] getAddScrutariDBNameArray() {
        Set<ScrutariDBName> set = new TreeSet<ScrutariDBName>();
        File[] dirs = uriDirectory.listFiles();
        int length = dirs.length;
        for (int i = 0; i < length; i++) {
            File dir = dirs[i];
            if (dir.isDirectory()) {
                String year = dir.getName();
                File[] files = dir.listFiles();
                int fileLength = files.length;
                for (int j = 0; j < fileLength; j++) {
                    File f = files[j];
                    String name = f.getName();
                    if (name.endsWith(".txt")) {
                        name = name.substring(0, name.length() - 4);
                        try {
                            ScrutariDBName scrutariDBName = ScrutariDBName.parse(name);
                            if (year.equals(scrutariDBName.getYear())) {
                                set.add(scrutariDBName);
                            }
                        } catch (ParseException pe) {
                        }
                    }
                }

            }
        }
        int size = set.size();
        int p = size - 1;
        ScrutariDBName[] result = new ScrutariDBName[size];
        for (ScrutariDBName scrutariDBName : set) {
            result[p] = scrutariDBName;
            p--;
        }
        return result;
    }

    @Override
    public CacheCheck checkLastCache(ScrutariDBName currentName) {
        SortedMap<ScrutariDBName, File> availableMap = new TreeMap<ScrutariDBName, File>();
        for (File dateCacheDir : cacheDirectory.listFiles()) {
            if (!dateCacheDir.isDirectory()) {
                continue;
            }
            try {
                ScrutariDBName scrutariDBName = ScrutariDBName.parse(dateCacheDir.getName());
                if ((currentName != null) && (scrutariDBName.compareTo(currentName) <= 0)) {
                    continue;
                }
                availableMap.put(scrutariDBName, dateCacheDir);
            } catch (ParseException pe) {
            }
        }
        if (availableMap.isEmpty()) {
            return null;
        }
        ScrutariDBName lastName = availableMap.lastKey();
        File dateCacheDir = availableMap.get(lastName);
        int version = CacheUtils.getCacheVersion(dateCacheDir);
        if (version != CacheUtils.CACHE_VERSION) {
            return null;
        }
        File scrutariDBDir = new File(dateCacheDir, "scrutaridb");
        DataCacheReaderFactory dataCacheReaderFactory = DirectoryDataCacheReaderFactory.loadFromCache(scrutariDBDir);
        if (dataCacheReaderFactory == null) {
            return null;
        }
        LexieCacheReaderFactory lexieCacheReaderFactory = DirectoryLexieCacheReaderFactory.loadFromCache(scrutariDBDir);
        if (lexieCacheReaderFactory == null) {
            return null;
        }
        return new InternalCacheCheck(lastName, dataCacheReaderFactory, lexieCacheReaderFactory);
    }

    private File getUriTreeFile(ScrutariDBName scrutariDBName) {
        if (scrutariDBName == null) {
            return new File(uriDirectory, "tree.txt");
        }
        File dir = new File(uriDirectory, scrutariDBName.getYear());
        dir.mkdirs();
        return new File(dir, scrutariDBName.getName() + ".txt");
    }

    private File getContentFile(String contentName) {
        if (contentName.equals(SctServerStorage.STATUS_NAME)) {
            return new File(dataDirectory, "status.txt");
        }
        return null;
    }

    private void convert() {
        File dir = new File(uriDirectory, "add");
        if (!dir.exists()) {
            return;
        }
        File[] files = dir.listFiles();
        int length = files.length;
        for (int i = 0; i < length; i++) {
            File f = files[i];
            if (f.isDirectory()) {
                continue;
            }
            String name = f.getName();
            if (name.endsWith(".txt")) {
                try {
                    ScrutariDBName scrutariDBName = ScrutariDBName.parse(name.substring(0, name.length() - 4));
                    File destDir = new File(uriDirectory, scrutariDBName.getYear());
                    destDir.mkdirs();
                    f.renameTo(new File(destDir, name));
                } catch (ParseException pe) {
                }
            }
        }
        try {
            IOUtils.forceDelete(dir);
        } catch (IOException ioe) {
        }
    }


    private static class InternalCacheCheck implements CacheCheck {

        private final ScrutariDBName scrutariDBName;
        private final DataCacheReaderFactory dataCacheReaderFactory;
        private final LexieCacheReaderFactory lexieCacheReaderFactory;

        private InternalCacheCheck(ScrutariDBName scrutariDBName, DataCacheReaderFactory dataCacheReaderFactory, LexieCacheReaderFactory lexieCacheReaderFactory) {
            this.scrutariDBName = scrutariDBName;
            this.dataCacheReaderFactory = dataCacheReaderFactory;
            this.lexieCacheReaderFactory = lexieCacheReaderFactory;
        }

        @Override
        public ScrutariDBName getScrutariDBName() {
            return scrutariDBName;
        }

        @Override
        public DataCacheReaderFactory getDataCacheReaderFactory() {
            return dataCacheReaderFactory;
        }

        @Override
        public LexieCacheReaderFactory getLexieCacheReaderFactory() {
            return lexieCacheReaderFactory;
        }

    }


    private static class InternalDataList implements DataList {

        private final File[] array;

        private InternalDataList(File[] array) {
            this.array = array;
        }

        @Override
        public int size() {
            return array.length;
        }

        @Override
        public InputStream getInputStream(int i) {
            try {
                return new FileInputStream(array[i]);
            } catch (IOException ioe) {
                throw new SctStorageException(ioe);
            }
        }

    }

}
