/* UtilLib - Copyright (c) 2014-2016 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import java.text.ParseException;
import java.util.Collection;
import java.util.LinkedHashSet;
import java.util.Set;
import net.mapeadores.util.text.StringUtils;


/**
 *
 * @author Vincent Calame
 */
public final class LangListUtils {

    public final static LangList EMPTY_LANGLIST = new EmptyLangList();

    private LangListUtils() {
    }

    public static LangList toCleanLangList(String s) {
        Lang[] array = toCleanLangArray(s);
        return new ArrayLangList(array);
    }

    public static Lang[] toCleanLangArray(String s) {
        String[] tokens = StringUtils.getTechnicalTokens(s, true);
        Set<Lang> langSet = new LinkedHashSet<Lang>();
        for (String token : tokens) {
            try {
                Lang lang = Lang.parse(token);
                langSet.add(lang);
            } catch (ParseException mce) {
            }
        }
        return langSet.toArray(new Lang[langSet.size()]);
    }

    public static String toString(LangList langList, String separator) {
        StringBuilder buf = new StringBuilder();
        int count = langList.getLangCount();
        for (int i = 0; i < count; i++) {
            if (i > 0) {
                buf.append(separator);
            }
            buf.append(langList.getLang(i).toString());
        }
        return buf.toString();
    }

    public static LangList singleton(Lang lang) {
        return new SingletonLangList(lang);
    }

    public static LangList fromArray(Lang[] array, boolean copy) {
        if (copy) {
            int length = array.length;
            Lang[] temp = new Lang[length];
            System.arraycopy(array, 0, temp, 0, length);
            array = temp;
        }
        return new ArrayLangList(array);
    }

    public static LangList fromCollection(Collection<Lang> collection) {
        Lang[] array = collection.toArray(new Lang[collection.size()]);
        return new ArrayLangList(array);
    }

    public static boolean isEqual(LangList langList, Lang[] array) {
        int length = array.length;
        if (length != langList.getLangCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            if (!array[i].equals(langList.getLang(i))) {
                return false;
            }
        }
        return true;
    }

    public static boolean isEqual(LangList langList, Collection<Lang> collection) {
        int length = collection.size();
        if (length != langList.getLangCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        int p = 0;
        for (Lang lang : collection) {
            if (!lang.equals(langList.getLang(p))) {
                return false;
            }
            p++;
        }
        return true;
    }

    public static boolean areEquals(LangList langList1, LangList langList2) {
        int length = langList2.getLangCount();
        if (length != langList1.getLangCount()) {
            return false;
        }
        if (length == 0) {
            return true;
        }
        for (int i = 0; i < length; i++) {
            if (!langList2.getLang(i).equals(langList1.getLang(i))) {
                return false;
            }
        }
        return true;
    }

    public static Lang getNearestLang(LangList langList, Lang lang) {
        int langCount = langList.getLangCount();
        if (langCount == 0) {
            throw new IllegalArgumentException("langList.getLangCount() == 0");
        }
        Lang option1 = null;
        Lang option2 = null;
        Lang rootLang = lang.getRootLang();
        for (int i = 0; i < langCount; i++) {
            Lang testedLang = langList.getLang(i);
            if (testedLang.equals(lang)) {
                return testedLang;
            } else if (testedLang.equals(rootLang)) {
                option1 = testedLang;
            } else if ((testedLang.getRootLang().equals(rootLang))) {
                option2 = testedLang;
            }
        }
        if (option1 != null) {
            return option1;
        }
        if (option2 != null) {
            return option2;
        }
        return langList.getLang(0);
    }


    private static class EmptyLangList implements LangList {

        private EmptyLangList() {
        }

        @Override
        public int getLangCount() {
            return 0;
        }

        @Override
        public Lang getLang(int i) {
            throw new IndexOutOfBoundsException("langCount = 0");
        }

    }


    private static class SingletonLangList implements LangList {

        private final Lang lang;

        private SingletonLangList(Lang lang) {
            this.lang = lang;
        }

        @Override
        public int getLangCount() {
            return 1;
        }

        @Override
        public Lang getLang(int i) {
            if (i != 0) {
                throw new IndexOutOfBoundsException("langCount = 1");
            }
            return lang;
        }

    }


    private static class ArrayLangList implements LangList {

        private final Lang[] array;

        private ArrayLangList(Lang[] array) {
            this.array = array;
        }

        @Override
        public int getLangCount() {
            return array.length;
        }

        @Override
        public Lang getLang(int i) {
            return array[i];
        }

    }

}
