/* UtilLib - Copyright (c) 2008-2014 Vincent Calame - Exemole
 * Logiciel libre donné sous triple licence :
 * 1) selon les termes de la CeCILL V2
 * 2) selon les termes de l’EUPL V.1.1
 * 3) selon les termes de la GNU GPLv3
 * Voir le fichier licences.txt
 */


package net.mapeadores.util.localisation;

import java.io.IOException;
import java.io.InputStream;
import java.text.ParseException;
import java.util.HashMap;
import java.util.List;
import java.util.Locale;
import java.util.Map;
import net.mapeadores.util.exceptions.InternalResourceException;
import net.mapeadores.util.ini.IniParser;


/**
 *
 * @author Vincent Calame
 */
public final class LocalisationUtils {

    public final static MessageList EMPTY_MESSAGELIST = new EmptyMessageList();
    private final static Lang FRENCH = Lang.build("fr");
    private final static Map<Lang, String> langVoMap;

    static {
        try {
            Map<String, String> iniMap = new HashMap<String, String>();
            InputStream langVoStream = LocalisationUtils.class.getResourceAsStream("vo_lang.ini");
            IniParser.parseIni(langVoStream, iniMap);
            langVoStream.close();
            langVoMap = new HashMap<Lang, String>();
            for (Map.Entry<String, String> entry : iniMap.entrySet()) {
                try {
                    Lang lang = Lang.parse(entry.getKey());
                    langVoMap.put(lang, entry.getValue());
                } catch (ParseException pe) {

                }
            }
        } catch (IOException ioe) {
            throw new InternalResourceException(ioe);
        }
    }

    private LocalisationUtils() {
    }

    public static String getKibiOctet(Locale locale) {
        String language = locale.getLanguage();
        if (language.equals("fr")) {
            return "Kio";
        } else {
            return "KiB";
        }
    }

    public static String getKibiOctet(Lang lang) {
        Lang rootLang = lang.getRootLang();
        if (rootLang.equals(FRENCH)) {
            return "Kio";
        } else {
            return "KiB";
        }
    }

    public static String getMebiOctet(Locale locale) {
        String language = locale.getLanguage();
        if (language.equals("fr")) {
            return "Mio";
        } else {
            return "MiB";
        }
    }

    public static String getMebiOctet(Lang lang) {
        Lang rootLang = lang.getRootLang();
        if (rootLang.equals(FRENCH)) {
            return "Mio";
        } else {
            return "MiB";
        }
    }

    public static String getVOTitle(Lang lang) {
        return langVoMap.get(lang);
    }

    public static Message toMessage(String messageKey) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        return new InternalMessage(messageKey, null);
    }

    public static Message toMessage(String messageKey, Object messageValue) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        return new InternalMessage(messageKey, messageValue);
    }

    public static MessageList toMessageList(List<Message> list) {
        if (list.isEmpty()) {
            return EMPTY_MESSAGELIST;
        }
        return new ArrayMessageList(list.toArray(new Message[list.size()]));
    }

    public static String cleanMessageKey(String messageKey) {
        if (messageKey.startsWith("_ ")) {
            messageKey = messageKey.substring(2);
        }
        return messageKey;
    }

    public static Lang getDefaultLang(LangContext langContext) {
        if (langContext instanceof ListLangContext) {
            return ((ListLangContext) langContext).getLang(0);
        } else if (langContext instanceof UserLangContext) {
            return ((UserLangContext) langContext).getWorkingLang();
        } else {
            throw new IllegalArgumentException("Unkown langContext implementation: " + langContext.getClass().getName());
        }
    }

    public static Locale getDefaultFormatLocale(LangContext langContext) {
        if (langContext instanceof ListLangContext) {
            return ((ListLangContext) langContext).getFormatLocale(0);
        } else if (langContext instanceof UserLangContext) {
            return ((UserLangContext) langContext).getFormatLocale();
        } else {
            return Locale.getDefault();
        }
    }

    public static UserLangContext toUserLangContext(Lang workingLang) {
        LangPreference langPreference = LangPreferenceBuilder.build(workingLang);
        Locale formatLocale = workingLang.toLocale();
        return new InternalUserLangContext(workingLang, langPreference, formatLocale);
    }

    public static UserLangContext toUserLangContext(Lang workingLang, LangPreference langPreference, Locale formatLocale) {
        if (formatLocale == null) {
            formatLocale = workingLang.toLocale();
        }
        return new InternalUserLangContext(workingLang, langPreference, formatLocale);
    }

    public static String toNotNullString(MessageLocalisation messageLocalisation, String messageKey, Object messageValue) {
        if (messageKey == null) {
            throw new IllegalArgumentException("messageKey is null");
        }
        if (messageKey.length() == 0) {
            throw new IllegalArgumentException("messageKey is empty");
        }
        String result = messageLocalisation.toString(messageKey, messageValue);
        if ((result == null) || (result.length() == 0)) {
            return messageKey;
        }
        return result;
    }


    private static class InternalUserLangContext implements UserLangContext {

        private final Lang workingLang;
        private final LangPreference langPreference;
        private final Locale formatLocale;

        private InternalUserLangContext(Lang workingLang, LangPreference langPreference, Locale formatLocale) {
            this.workingLang = workingLang;
            this.langPreference = langPreference;
            this.formatLocale = formatLocale;
        }

        @Override
        public Lang getWorkingLang() {
            return workingLang;
        }

        @Override
        public LangPreference getLangPreference() {
            return langPreference;
        }

        @Override
        public Locale getFormatLocale() {
            return formatLocale;
        }

    }


    private static class EmptyMessageList implements MessageList {

        private EmptyMessageList() {
        }

        @Override
        public int getMessageCount() {
            return 0;
        }

        @Override
        public Message getMessage(int i) {
            throw new IndexOutOfBoundsException();
        }

    }


    private static class ArrayMessageList implements MessageList {

        private final Message[] array;

        private ArrayMessageList(Message[] array) {
            this.array = array;
        }

        @Override
        public int getMessageCount() {
            return array.length;
        }

        @Override
        public Message getMessage(int i) {
            return array[i];
        }

    }


    private static class InternalMessage implements Message {

        private final String messageKey;
        private final Object messageValue;

        private InternalMessage(String messageKey, Object messageValue) {
            this.messageKey = messageKey;
            this.messageValue = messageValue;
        }

        @Override
        public String getMessageKey() {
            return messageKey;
        }

        @Override
        public Object getMessageValue() {
            return messageValue;
        }

    }

}
